"""
// @author: ComPleHN
// @file: page.vue
// @time: 2025/5/28 12:49
// @description: 本文件是对于echarts的 3D折线图 图表测试 
"""
# ---------------------------------------------- Page - Page_draggable_layout ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Grid, Line, Liquid, Page, Pie
from pyecharts.commons.utils import JsCode
from pyecharts.components import Table
from pyecharts.faker import Faker


def bar_datazoom_slider() -> Bar:
    c = (
        Bar()
        .add_xaxis(Faker.days_attrs)
        .add_yaxis("商家A", Faker.days_values)
        .set_global_opts(
            title_opts=opts.TitleOpts(title="Bar-DataZoom（slider-水平）"),
            datazoom_opts=[opts.DataZoomOpts()],
        )
    )
    return c


def line_markpoint() -> Line:
    c = (
        Line()
        .add_xaxis(Faker.choose())
        .add_yaxis(
            "商家A",
            Faker.values(),
            markpoint_opts=opts.MarkPointOpts(data=[opts.MarkPointItem(type_="min")]),
        )
        .add_yaxis(
            "商家B",
            Faker.values(),
            markpoint_opts=opts.MarkPointOpts(data=[opts.MarkPointItem(type_="max")]),
        )
        .set_global_opts(title_opts=opts.TitleOpts(title="Line-MarkPoint"))
    )
    return c


def pie_rosetype() -> Pie:
    v = Faker.choose()
    c = (
        Pie()
        .add(
            "",
            [list(z) for z in zip(v, Faker.values())],
            radius=["30%", "75%"],
            center=["25%", "50%"],
            rosetype="radius",
            label_opts=opts.LabelOpts(is_show=False),
        )
        .add(
            "",
            [list(z) for z in zip(v, Faker.values())],
            radius=["30%", "75%"],
            center=["75%", "50%"],
            rosetype="area",
        )
        .set_global_opts(title_opts=opts.TitleOpts(title="Pie-玫瑰图示例"))
    )
    return c


def grid_mutil_yaxis() -> Grid:
    x_data = ["{}月".format(i) for i in range(1, 13)]
    bar = (
        Bar()
        .add_xaxis(x_data)
        .add_yaxis(
            "蒸发量",
            [2.0, 4.9, 7.0, 23.2, 25.6, 76.7, 135.6, 162.2, 32.6, 20.0, 6.4, 3.3],
            yaxis_index=0,
            color="#d14a61",
        )
        .add_yaxis(
            "降水量",
            [2.6, 5.9, 9.0, 26.4, 28.7, 70.7, 175.6, 182.2, 48.7, 18.8, 6.0, 2.3],
            yaxis_index=1,
            color="#5793f3",
        )
        .extend_axis(
            yaxis=opts.AxisOpts(
                name="蒸发量",
                type_="value",
                min_=0,
                max_=250,
                position="right",
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color="#d14a61")
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
            )
        )
        .extend_axis(
            yaxis=opts.AxisOpts(
                type_="value",
                name="温度",
                min_=0,
                max_=25,
                position="left",
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color="#675bba")
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value} °C"),
                splitline_opts=opts.SplitLineOpts(
                    is_show=True, linestyle_opts=opts.LineStyleOpts(opacity=1)
                ),
            )
        )
        .set_global_opts(
            yaxis_opts=opts.AxisOpts(
                name="降水量",
                min_=0,
                max_=250,
                position="right",
                offset=80,
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color="#5793f3")
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
            ),
            title_opts=opts.TitleOpts(title="Grid-多 Y 轴示例"),
            tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        )
    )

    line = (
        Line()
        .add_xaxis(x_data)
        .add_yaxis(
            "平均温度",
            [2.0, 2.2, 3.3, 4.5, 6.3, 10.2, 20.3, 23.4, 23.0, 16.5, 12.0, 6.2],
            yaxis_index=2,
            color="#675bba",
            label_opts=opts.LabelOpts(is_show=False),
        )
    )

    bar.overlap(line)
    return Grid().add(
        bar, opts.GridOpts(pos_left="5%", pos_right="20%"), is_control_axis_index=True
    )


def liquid_data_precision() -> Liquid:
    c = (
        Liquid()
        .add(
            "lq",
            [0.3254],
            label_opts=opts.LabelOpts(
                font_size=50,
                formatter=JsCode(
                    """function (param) {
                        return (Math.floor(param.value * 10000) / 100) + '%';
                    }"""
                ),
                position="inside",
            ),
        )
        .set_global_opts(title_opts=opts.TitleOpts(title="Liquid-数据精度"))
    )
    return c


def table_base() -> Table:
    table = Table()

    headers = ["City name", "Area", "Population", "Annual Rainfall"]
    rows = [
        ["Brisbane", 5905, 1857594, 1146.4],
        ["Adelaide", 1295, 1158259, 600.5],
        ["Darwin", 112, 120900, 1714.7],
        ["Hobart", 1357, 205556, 619.5],
        ["Sydney", 2058, 4336374, 1214.8],
        ["Melbourne", 1566, 3806092, 646.9],
        ["Perth", 5386, 1554769, 869.4],
    ]
    table.add(headers, rows).set_global_opts(
        title_opts=opts.ComponentTitleOpts(title="Table")
    )
    return table


def page_draggable_layout():
    page = Page(layout=Page.DraggablePageLayout)
    page.add(
        bar_datazoom_slider(),
        line_markpoint(),
        pie_rosetype(),
        grid_mutil_yaxis(),
        liquid_data_precision(),
        table_base(),
    )
    return page

c1 = page_draggable_layout()

# ---------------------------------------------- Page - Page_simple_layout ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Grid, Line, Liquid, Page, Pie
from pyecharts.commons.utils import JsCode
from pyecharts.components import Table
from pyecharts.faker import Faker


def bar_datazoom_slider() -> Bar:
    c = (
        Bar()
        .add_xaxis(Faker.days_attrs)
        .add_yaxis("商家A", Faker.days_values)
        .set_global_opts(
            title_opts=opts.TitleOpts(title="Bar-DataZoom（slider-水平）"),
            datazoom_opts=[opts.DataZoomOpts()],
        )
    )
    return c


def line_markpoint() -> Line:
    c = (
        Line()
        .add_xaxis(Faker.choose())
        .add_yaxis(
            "商家A",
            Faker.values(),
            markpoint_opts=opts.MarkPointOpts(data=[opts.MarkPointItem(type_="min")]),
        )
        .add_yaxis(
            "商家B",
            Faker.values(),
            markpoint_opts=opts.MarkPointOpts(data=[opts.MarkPointItem(type_="max")]),
        )
        .set_global_opts(title_opts=opts.TitleOpts(title="Line-MarkPoint"))
    )
    return c


def pie_rosetype() -> Pie:
    v = Faker.choose()
    c = (
        Pie()
        .add(
            "",
            [list(z) for z in zip(v, Faker.values())],
            radius=["30%", "75%"],
            center=["25%", "50%"],
            rosetype="radius",
            label_opts=opts.LabelOpts(is_show=False),
        )
        .add(
            "",
            [list(z) for z in zip(v, Faker.values())],
            radius=["30%", "75%"],
            center=["75%", "50%"],
            rosetype="area",
        )
        .set_global_opts(title_opts=opts.TitleOpts(title="Pie-玫瑰图示例"))
    )
    return c


def grid_mutil_yaxis() -> Grid:
    x_data = ["{}月".format(i) for i in range(1, 13)]
    bar = (
        Bar()
        .add_xaxis(x_data)
        .add_yaxis(
            "蒸发量",
            [2.0, 4.9, 7.0, 23.2, 25.6, 76.7, 135.6, 162.2, 32.6, 20.0, 6.4, 3.3],
            yaxis_index=0,
            color="#d14a61",
        )
        .add_yaxis(
            "降水量",
            [2.6, 5.9, 9.0, 26.4, 28.7, 70.7, 175.6, 182.2, 48.7, 18.8, 6.0, 2.3],
            yaxis_index=1,
            color="#5793f3",
        )
        .extend_axis(
            yaxis=opts.AxisOpts(
                name="蒸发量",
                type_="value",
                min_=0,
                max_=250,
                position="right",
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color="#d14a61")
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
            )
        )
        .extend_axis(
            yaxis=opts.AxisOpts(
                type_="value",
                name="温度",
                min_=0,
                max_=25,
                position="left",
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color="#675bba")
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value} °C"),
                splitline_opts=opts.SplitLineOpts(
                    is_show=True, linestyle_opts=opts.LineStyleOpts(opacity=1)
                ),
            )
        )
        .set_global_opts(
            yaxis_opts=opts.AxisOpts(
                name="降水量",
                min_=0,
                max_=250,
                position="right",
                offset=80,
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color="#5793f3")
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
            ),
            title_opts=opts.TitleOpts(title="Grid-多 Y 轴示例"),
            tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        )
    )

    line = (
        Line()
        .add_xaxis(x_data)
        .add_yaxis(
            "平均温度",
            [2.0, 2.2, 3.3, 4.5, 6.3, 10.2, 20.3, 23.4, 23.0, 16.5, 12.0, 6.2],
            yaxis_index=2,
            color="#675bba",
            label_opts=opts.LabelOpts(is_show=False),
        )
    )

    bar.overlap(line)
    return Grid().add(
        bar, opts.GridOpts(pos_left="5%", pos_right="20%"), is_control_axis_index=True
    )


def liquid_data_precision() -> Liquid:
    c = (
        Liquid()
        .add(
            "lq",
            [0.3254],
            label_opts=opts.LabelOpts(
                font_size=50,
                formatter=JsCode(
                    """function (param) {
                        return (Math.floor(param.value * 10000) / 100) + '%';
                    }"""
                ),
                position="inside",
            ),
        )
        .set_global_opts(title_opts=opts.TitleOpts(title="Liquid-数据精度"))
    )
    return c


def table_base() -> Table:
    table = Table()

    headers = ["City name", "Area", "Population", "Annual Rainfall"]
    rows = [
        ["Brisbane", 5905, 1857594, 1146.4],
        ["Adelaide", 1295, 1158259, 600.5],
        ["Darwin", 112, 120900, 1714.7],
        ["Hobart", 1357, 205556, 619.5],
        ["Sydney", 2058, 4336374, 1214.8],
        ["Melbourne", 1566, 3806092, 646.9],
        ["Perth", 5386, 1554769, 869.4],
    ]
    table.add(headers, rows).set_global_opts(
        title_opts=opts.ComponentTitleOpts(title="Table")
    )
    return table


def page_simple_layout():
    page = Page(layout=Page.SimplePageLayout)
    page.add(
        bar_datazoom_slider(),
        line_markpoint(),
        pie_rosetype(),
        grid_mutil_yaxis(),
        liquid_data_precision(),
        table_base(),
    )
    return page

c2 = page_simple_layout()

# ---------------------------------------------- Page - Page_default_layout ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Grid, Line, Liquid, Page, Pie
from pyecharts.commons.utils import JsCode
from pyecharts.components import Table
from pyecharts.faker import Faker


def bar_datazoom_slider() -> Bar:
    c = (
        Bar()
        .add_xaxis(Faker.days_attrs)
        .add_yaxis("商家A", Faker.days_values)
        .set_global_opts(
            title_opts=opts.TitleOpts(title="Bar-DataZoom（slider-水平）"),
            datazoom_opts=[opts.DataZoomOpts()],
        )
    )
    return c


def line_markpoint() -> Line:
    c = (
        Line()
        .add_xaxis(Faker.choose())
        .add_yaxis(
            "商家A",
            Faker.values(),
            markpoint_opts=opts.MarkPointOpts(data=[opts.MarkPointItem(type_="min")]),
        )
        .add_yaxis(
            "商家B",
            Faker.values(),
            markpoint_opts=opts.MarkPointOpts(data=[opts.MarkPointItem(type_="max")]),
        )
        .set_global_opts(title_opts=opts.TitleOpts(title="Line-MarkPoint"))
    )
    return c


def pie_rosetype() -> Pie:
    v = Faker.choose()
    c = (
        Pie()
        .add(
            "",
            [list(z) for z in zip(v, Faker.values())],
            radius=["30%", "75%"],
            center=["25%", "50%"],
            rosetype="radius",
            label_opts=opts.LabelOpts(is_show=False),
        )
        .add(
            "",
            [list(z) for z in zip(v, Faker.values())],
            radius=["30%", "75%"],
            center=["75%", "50%"],
            rosetype="area",
        )
        .set_global_opts(title_opts=opts.TitleOpts(title="Pie-玫瑰图示例"))
    )
    return c


def grid_mutil_yaxis() -> Grid:
    x_data = ["{}月".format(i) for i in range(1, 13)]
    bar = (
        Bar()
        .add_xaxis(x_data)
        .add_yaxis(
            "蒸发量",
            [2.0, 4.9, 7.0, 23.2, 25.6, 76.7, 135.6, 162.2, 32.6, 20.0, 6.4, 3.3],
            yaxis_index=0,
            color="#d14a61",
        )
        .add_yaxis(
            "降水量",
            [2.6, 5.9, 9.0, 26.4, 28.7, 70.7, 175.6, 182.2, 48.7, 18.8, 6.0, 2.3],
            yaxis_index=1,
            color="#5793f3",
        )
        .extend_axis(
            yaxis=opts.AxisOpts(
                name="蒸发量",
                type_="value",
                min_=0,
                max_=250,
                position="right",
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color="#d14a61")
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
            )
        )
        .extend_axis(
            yaxis=opts.AxisOpts(
                type_="value",
                name="温度",
                min_=0,
                max_=25,
                position="left",
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color="#675bba")
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value} °C"),
                splitline_opts=opts.SplitLineOpts(
                    is_show=True, linestyle_opts=opts.LineStyleOpts(opacity=1)
                ),
            )
        )
        .set_global_opts(
            yaxis_opts=opts.AxisOpts(
                name="降水量",
                min_=0,
                max_=250,
                position="right",
                offset=80,
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color="#5793f3")
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
            ),
            title_opts=opts.TitleOpts(title="Grid-多 Y 轴示例"),
            tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        )
    )

    line = (
        Line()
        .add_xaxis(x_data)
        .add_yaxis(
            "平均温度",
            [2.0, 2.2, 3.3, 4.5, 6.3, 10.2, 20.3, 23.4, 23.0, 16.5, 12.0, 6.2],
            yaxis_index=2,
            color="#675bba",
            label_opts=opts.LabelOpts(is_show=False),
        )
    )

    bar.overlap(line)
    return Grid().add(
        bar, opts.GridOpts(pos_left="5%", pos_right="20%"), is_control_axis_index=True
    )


def liquid_data_precision() -> Liquid:
    c = (
        Liquid()
        .add(
            "lq",
            [0.3254],
            label_opts=opts.LabelOpts(
                font_size=50,
                formatter=JsCode(
                    """function (param) {
                        return (Math.floor(param.value * 10000) / 100) + '%';
                    }"""
                ),
                position="inside",
            ),
        )
        .set_global_opts(title_opts=opts.TitleOpts(title="Liquid-数据精度"))
    )
    return c


def table_base() -> Table:
    table = Table()

    headers = ["City name", "Area", "Population", "Annual Rainfall"]
    rows = [
        ["Brisbane", 5905, 1857594, 1146.4],
        ["Adelaide", 1295, 1158259, 600.5],
        ["Darwin", 112, 120900, 1714.7],
        ["Hobart", 1357, 205556, 619.5],
        ["Sydney", 2058, 4336374, 1214.8],
        ["Melbourne", 1566, 3806092, 646.9],
        ["Perth", 5386, 1554769, 869.4],
    ]
    table.add(headers, rows).set_global_opts(
        title_opts=opts.ComponentTitleOpts(title="Table")
    )
    return table


def page_default_layout():
    page = Page()
    page.add(
        bar_datazoom_slider(),
        line_markpoint(),
        pie_rosetype(),
        grid_mutil_yaxis(),
        liquid_data_precision(),
        table_base(),
    )
    return page

c3 = page_default_layout()

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="page",
    public=True,
)

swanlab.log(
    {
        "Page - Page_draggable_layout": c1,
        "Page - Page_simple_layout": c2,
        "Page - Page_default_layout": c3
    }
)


"""
@author: cunyue
@file: __init__.py
@time: 2025/6/21 17:10
@description: 回调器模块，四大回调器对应 swanlab 的四种运行模式。
1. local： 本地模式回调器，此回调不自动导出
2. cloud： 云端模式回调器
3. offline： 离线模式回调器
4. disabled： 禁用回调器
"""

from .cloud import CloudPyCallback
from .disabled import DisabledCallback
from .offline import OfflineCallback



import wandb
import random
import swanlab
import numpy as np

swanlab.sync_wandb(wandb_run=False)

wandb.init(
  project="test",
  config={"a": 1, "b": 2},
  name="test",
  notes="test_wandb_sync",
  tags=["test", "wandb_sync"],
  )

wandb.config.update({"c": 3, "d": 4})
print(swanlab.config.get("c"))

wb_image = wandb.Image(np.random.randint(low=0, high=256, size=(100, 100, 3), dtype=np.uint8), caption="test_image")
wb_image_list = [wb_image] * 2

wandb.log({"im": wb_image_list}, step=10)

epochs = 10
offset = random.random() / 5
for epoch in range(2, epochs):
  acc = 1 - 2 ** -epoch - random.random() / epoch - offset
  loss = 2 ** -epoch + random.random() / epoch + offset

  wandb.log({"acc": acc, "loss": loss})


"""
@author: cunyue
@file: test_cli_verify.py
@time: 2025/9/23 11:22
@description: 测试cli verify命令
"""

import os

import pytest
from click.testing import CliRunner

import tutils as T
from swanlab.cli.main import cli


# noinspection PyTypeChecker
@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_verify_ok():
    runner = CliRunner()
    result = runner.invoke(cli, ["verify"])
    assert result.exit_code == 0
    assert ("You are logged into " + T.WEB_HOST) in result.output


# noinspection PyTypeChecker
def test_verify_error():
    runner = CliRunner()
    if T.SwanLabEnv.API_KEY.value in os.environ:
        del os.environ[T.SwanLabEnv.API_KEY.value]
    result = runner.invoke(cli, ["verify"])
    assert result.exit_code == 1
    assert 'You are not verified. Please use `swanlab login` to login.' in result.output



"""
@author: cunyue
@file: __init__.py
@time: 2025/6/19 16:05
@description: 全局、用户可访问工具依赖
"""

from .callback import *
from .logger import *
from .models.config import *
from .models.data import *
from .models.log import *
from .models.metric import *



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/4/1 15:58
@File: start_thread.py.py
@IDE: pycharm
@Description:
    生成线程池，生成通信管道
"""
import threading
import time
from queue import Queue
from typing import List, Tuple, Callable, Dict

from swanlab.log import swanlog
from .log_collector import LogCollectorTask
from .utils import LogQueue, TimerFlag
from .utils import ThreadUtil


class ThreadPool:
    """
    线程池类，负责管理线程和为线程生成通信管道
    只生成一个管道，用于其他线程与主要线程的通信
    """

    SLEEP_TIME = 1
    """
    默认的任务休眠时间，单位秒
    """
    UPLOAD_THREAD_NAME = "MsgUploader"
    """
    数据上传线程的名称
    """

    def __init__(self, upload_sleep_time: float = None):
        self.thread_pool = {}
        # 日志聚合器
        self.collector = LogCollectorTask()
        # timer集合
        self.thread_timer: Dict[str, TimerFlag] = {}
        self.__callbacks: List[Callable] = []
        self.__queue = Queue()
        # 生成数据上传线程，此线程包含聚合器和数据上传任务，负责收集所有线程向主线程发送的日志信息
        self.upload_thread = self.create_thread(
            target=self.collector.task,
            args=(),
            name=self.UPLOAD_THREAD_NAME,
            sleep_time=upload_sleep_time,
            callback=self.collector.callback,
        )

        self.queue = LogQueue(queue=self.__queue, readable=False, writable=True)
        """
        一个线程安全的队列，用于主线程向数据上传线程通信
        """

    def create_thread(
        self,
        target: Callable,
        args: Tuple = (),
        name: str = None,
        sleep_time: float = None,
        callback: Callable = None,
    ) -> threading.Thread:
        """
        创建一个线程
        :param target: 线程任务，约定传入的线程任务的第一个参数为 ThreadUtil 的实例
        :param args: 任务参数
        :param name: 线程名称
        :param sleep_time: 任务休眠时间
        :param callback: 线程结束时的回调函数
        :return: 线程对象
        """
        if name is None:
            name = f"Thread-{len(self.thread_pool)}"
        if name in self.thread_pool:
            raise Exception(f"Thread name {name} already exists")
        if sleep_time is None:
            sleep_time = self.SLEEP_TIME
        if name == self.UPLOAD_THREAD_NAME:
            q = LogQueue(queue=self.__queue, readable=True, writable=False)
        else:
            q = LogQueue(queue=self.__queue, readable=False, writable=True)
        thread_util = ThreadUtil(q, name)
        callback = ThreadUtil.wrapper_callback(callback, (thread_util, *args)) if callback is not None else None
        task = self._create_loop(name, sleep_time, target, (thread_util, *args))
        thread = threading.Thread(target=task, daemon=True, name=name)
        self.thread_pool[name] = thread
        if callback is not None:
            self.__callbacks.append(callback)
        thread.start()
        return thread

    @property
    def sub_threads(self):
        """
        除了日志上传线程外的所有线程
        """
        return {name: thread for name, thread in self.thread_pool.items() if name != self.UPLOAD_THREAD_NAME}

    def _create_loop(self, name: str, sleep_time: float, task: Callable, args: Tuple[ThreadUtil, ...]) -> [Callable]:
        """
        创建一个事件循环，循环执行传入线程池的任务
        :param name: 线程名称
        :param sleep_time: 任务休眠时间
        :param task: 任务
        :param args: 任务参数
        :return: 新的事件循环函数，用于启动新的线程
        """
        timer: TimerFlag = args[0].timer
        self.thread_timer[name] = timer

        # 新的执行函数，执行任务后等待sleep_time时间后再重新执行
        def new_task():
            while True:
                swanlog.debug(f"{threading.current_thread().name} is running")
                # 如果task是同步函数，直接调用，否则使用await调用
                task(*args)
                time.sleep(sleep_time)
                # 如果定时器停止，则退出循环
                if not timer.running:
                    return swanlog.debug(f"{threading.current_thread().name} is stopped")

        return new_task

    def finish(self):
        """
        [在主线程中] 结束线程池中的所有线程，并执行所有线程的结束任务
        """
        # print("线程池准备停止")
        # 第一步停止所有非日志上传线程
        for name, _ in self.thread_pool.items():
            self.thread_timer[name].cancel()

        # 由于线程池中的线程是daemon线程，并且已经cancel()，因此不需要等待，因为不会再有新的消息产生
        # for _, thread in self.thread_pool.items():
        #     thread.join()
        #     # print(f"{thread.name} is stopped")

        # print("非日志上传线程结束")
        # 倒序执行回调函数

        [cb() for cb in self.__callbacks[::-1]]
        # 停止日志上传线程的任务
        self.thread_timer[self.UPLOAD_THREAD_NAME].cancel()
        # 等待日志上传线程结束
        self.upload_thread.join()
        # print("线程池结束")



"""
@author: cunyue
@file: __init__.py
@time: 2025/9/23 17:00
@description: swanlab mode command，等价于自动填写 swanlab.init(mode=xxx)
具体方案是在 swanlog 文件夹下创建 settings 文件，依据 INI 格式存储配置信息，这部分业务存在swanlab.swanlab_settings中
"""

import click

from swanlab.env import create_swanlog_dir
from swanlab.swanlab_settings import write_folder_settings


@click.command()
def disabled():
    """Disable SwanLab"""
    logdir = create_swanlog_dir()
    write_folder_settings(logdir, default={'mode': 'disabled'})


@click.command()
def local():
    """Use local mode for SwanLab"""
    logdir = create_swanlog_dir()
    write_folder_settings(logdir, default={'mode': 'local'})


@click.command()
def online():
    """Use cloud mode for SwanLab"""
    logdir = create_swanlog_dir()
    write_folder_settings(logdir, default={'mode': 'cloud'})


@click.command()
def offline():
    """Use offline mode for SwanLab"""
    logdir = create_swanlog_dir()
    write_folder_settings(logdir, default={'mode': 'offline'})


__all__ = ['disabled', 'local', 'online', 'offline']



"""
// @author: ComPleHN
// @file: scatter.vue
// @time: 2025/5/29 10:32
// @description: 本文件是对于echarts的 散点图 图表的测试
"""
# ---------------------------------------------- Scatter - Basic_scatter_chart ----------------------------------------------
import pyecharts.options as opts
from pyecharts.charts import Scatter

"""
Gallery 使用 pyecharts 1.1.0
参考地址: https://echarts.apache.org/examples/editor.html?c=scatter-simple

目前无法实现的功能:

1、暂无
"""

data = [
    [10.0, 8.04],
    [8.0, 6.95],
    [13.0, 7.58],
    [9.0, 8.81],
    [11.0, 8.33],
    [14.0, 9.96],
    [6.0, 7.24],
    [4.0, 4.26],
    [12.0, 10.84],
    [7.0, 4.82],
    [5.0, 5.68],
]
data.sort(key=lambda x: x[0])
x_data = [d[0] for d in data]
y_data = [d[1] for d in data]

c1 = (
    Scatter()
    .add_xaxis(xaxis_data=x_data)
    .add_yaxis(
        series_name="",
        y_axis=y_data,
        symbol_size=20,
        label_opts=opts.LabelOpts(is_show=False),
    )
    .set_series_opts()
    .set_global_opts(
        xaxis_opts=opts.AxisOpts(
            type_="value", splitline_opts=opts.SplitLineOpts(is_show=True)
        ),
        yaxis_opts=opts.AxisOpts(
            type_="value",
            axistick_opts=opts.AxisTickOpts(is_show=True),
            splitline_opts=opts.SplitLineOpts(is_show=True),
        ),
        tooltip_opts=opts.TooltipOpts(is_show=False),
    )
)

# ---------------------------------------------- Scatter - Scatter_multi_dimension ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Scatter
from pyecharts.commons.utils import JsCode
from pyecharts.faker import Faker

c2 = (
    Scatter()
    .add_xaxis(Faker.choose())
    .add_yaxis(
        "商家A",
        [list(z) for z in zip(Faker.values(), Faker.choose())],
        label_opts=opts.LabelOpts(
            formatter=JsCode(
                "function(params){return params.value[1] +' : '+ params.value[2];}"
            )
        ),
    )
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Scatter-多维度数据"),
        tooltip_opts=opts.TooltipOpts(
            formatter=JsCode(
                "function (params) {return params.name + ' : ' + params.value[2];}"
            )
        ),
        visualmap_opts=opts.VisualMapOpts(
            type_="color", max_=150, min_=20, dimension=1
        ),
    )
)

# ----------------------------------------------Scatter - Scatter_visualmap_color  ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Scatter
from pyecharts.faker import Faker

c3 = (
    Scatter()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Scatter-VisualMap(Color)"),
        visualmap_opts=opts.VisualMapOpts(max_=150),
    )
)

# ---------------------------------------------- Scatter - Scatter_splitline ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Scatter
from pyecharts.faker import Faker

c4 = (
    Scatter()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Scatter-显示分割线"),
        xaxis_opts=opts.AxisOpts(splitline_opts=opts.SplitLineOpts(is_show=True)),
        yaxis_opts=opts.AxisOpts(splitline_opts=opts.SplitLineOpts(is_show=True)),
    )
)

# ---------------------------------------------- Scatter - Scatter_visualmap_size ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Scatter
from pyecharts.faker import Faker

c5 = (
    Scatter()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Scatter-VisualMap(Size)"),
        visualmap_opts=opts.VisualMapOpts(type_="size", max_=150, min_=20),
    )
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="scatter",
    public=True,
)

swanlab.log(
    {
        "Scatter - Basic_scatter_chart": c1,
        "Scatter - Scatter_multi_dimension": c2,
        "Scatter - Scatter_visualmap_color": c3,
        "Scatter - Scatter_splitline": c4,
        "Scatter - Scatter_visualmap_size": c5
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2025/4/30 12:11
@File: group.py
@IDE: pycharm
@Description:
    组织相关的开放API
"""

from swanlab.api.base import ApiBase, ApiHTTP
from swanlab.api.types import ApiResponse


class GroupAPI(ApiBase):
    def __init__(self, http: ApiHTTP):
        super().__init__(http)

    def list_workspaces(self) -> ApiResponse[list]:
        resp = self.http.get("/group/", params={})
        if resp.errmsg:
            return resp
        groups = resp.data.get("list", [])
        resp.data = [
            {
                "name": item["name"],
                "username": item["username"],
                "role": item["role"]
            }
            for item in groups
        ]
        return resp



"""
// @author: ComPleHN
// @file: timeline.vue
// @time: 2025/5/29 10:52
// @description: 本文件是对于echarts的 时间轴组件 图表的测试
"""
# ---------------------------------------------- Timeline - Timeline_pie ---------------------------------------------- 
from pyecharts import options as opts
from pyecharts.charts import Pie, Timeline
from pyecharts.faker import Faker

attr = Faker.choose()
c1 = Timeline()
for i in range(2015, 2020):
    pie = (
        Pie()
        .add(
            "商家A",
            [list(z) for z in zip(attr, Faker.values())],
            rosetype="radius",
            radius=["30%", "55%"],
        )
        .set_global_opts(title_opts=opts.TitleOpts("某商店{}年营业额".format(i)))
    )
    c1.add(pie, "{}年".format(i))

# ---------------------------------------------- Timeline - Timeline_bar_with_graphic ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Timeline
from pyecharts.commons.utils import JsCode
from pyecharts.faker import Faker

x = Faker.choose()
c2 = Timeline()
for i in range(2015, 2020):
    bar = (
        Bar()
        .add_xaxis(x)
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .set_global_opts(
            title_opts=opts.TitleOpts("某商店{}年营业额 - With Graphic 组件".format(i)),
            graphic_opts=[
                opts.GraphicGroup(
                    graphic_item=opts.GraphicItem(
                        rotation=JsCode("Math.PI / 4"),
                        bounding="raw",
                        right=100,
                        bottom=110,
                        z=100,
                    ),
                    children=[
                        opts.GraphicRect(
                            graphic_item=opts.GraphicItem(
                                left="center", top="center", z=100
                            ),
                            graphic_shape_opts=opts.GraphicShapeOpts(
                                width=400, height=50
                            ),
                            graphic_basicstyle_opts=opts.GraphicBasicStyleOpts(
                                fill="rgba(0,0,0,0.3)"
                            ),
                        ),
                        opts.GraphicText(
                            graphic_item=opts.GraphicItem(
                                left="center", top="center", z=100
                            ),
                            graphic_textstyle_opts=opts.GraphicTextStyleOpts(
                                text="某商店{}年营业额".format(i),
                                font="bold 26px Microsoft YaHei",
                                graphic_basicstyle_opts=opts.GraphicBasicStyleOpts(
                                    fill="#fff"
                                ),
                            ),
                        ),
                    ],
                )
            ],
        )
    )
    c2.add(bar, "{}年".format(i))

# ---------------------------------------------- Timeline - Timeline_bmap ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import BMap, Timeline
from pyecharts.faker import Faker

c3 = Timeline()
c3.add_schema(pos_left="50%", pos_right="10px", pos_bottom="15px")
for i in range(2015, 2020):
    bmap = (
        BMap()
        .add_schema(baidu_ak="FAKE_AK", center=[120.13066322374, 30.240018034923])
        .add(
            "bmap",
            [list(z) for z in zip(Faker.provinces, Faker.values())],
            type_="heatmap",
        )
        .set_global_opts(
            title_opts=opts.TitleOpts(title="Timeline-BMap-热力图-{}年".format(i)),
            visualmap_opts=opts.VisualMapOpts(pos_bottom="center", pos_right="10px"),
            tooltip_opts=opts.TooltipOpts(formatter=None),
        )
    )
    c3.add(bmap, "{}年".format(i))

# ---------------------------------------------- Timeline - Timeline_sankey ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Sankey, Timeline
from pyecharts.faker import Faker

c4 = Timeline()
names = ("商家A", "商家B", "商家C")
nodes = [{"name": name} for name in names]
for i in range(2015, 2020):
    links = [
        {"source": names[0], "target": names[1], "value": Faker.values()[0]},
        {"source": names[1], "target": names[2], "value": Faker.values()[0]},
    ]
    sankey = (
        Sankey()
        .add(
            "sankey",
            nodes,
            links,
            linestyle_opt=opts.LineStyleOpts(opacity=0.2, curve=0.5, color="source"),
            label_opts=opts.LabelOpts(position="right"),
        )
        .set_global_opts(
            title_opts=opts.TitleOpts(title="{}年商店（A, B, C）营业额差".format(i))
        )
    )
    c4.add(sankey, "{}年".format(i))
    
# ---------------------------------------------- Timeline - Timeline_bar_reversal ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Timeline
from pyecharts.faker import Faker

c5 = Timeline()
for i in range(2015, 2020):
    bar = (
        Bar()
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values(), label_opts=opts.LabelOpts(position="right"))
        .add_yaxis("商家B", Faker.values(), label_opts=opts.LabelOpts(position="right"))
        .reversal_axis()
        .set_global_opts(
            title_opts=opts.TitleOpts("Timeline-Bar-Reversal (时间: {} 年)".format(i))
        )
    )
    c5.add(bar, "{}年".format(i))

# ---------------------------------------------- Timeline - Timeline_bar ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Timeline
from pyecharts.faker import Faker

x = Faker.choose()
c6 = Timeline()
for i in range(2015, 2020):
    bar = (
        Bar()
        .add_xaxis(x)
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("某商店{}年营业额".format(i)))
    )
    c6.add(bar, "{}年".format(i))

# ---------------------------------------------- Timeline - Timeline_multi_axis ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Timeline
from pyecharts.faker import Faker

c7 = Timeline()
for i in range(2015, 2020):
    bar = (
        Bar()
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("某商店{}年营业额".format(i)))
    )
    c7.add(bar, "{}年".format(i))
    
# ---------------------------------------------- Timeline - Timeline_map ---------------------------------------------- 
from pyecharts import options as opts
from pyecharts.charts import Map, Timeline
from pyecharts.faker import Faker

c8 = Timeline()
for i in range(2015, 2020):
    map0 = (
        Map()
        .add("商家A", [list(z) for z in zip(Faker.provinces, Faker.values())], "china")
        .set_global_opts(
            title_opts=opts.TitleOpts(title="Map-{}年某些数据".format(i)),
            visualmap_opts=opts.VisualMapOpts(max_=200),
        )
    )
    c8.add(map0, "{}年".format(i))

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="timeline",
    public=True,
)

swanlab.log(
    {
        "Timeline - Timeline_pie": c1,
        "Timeline - Timeline_bar_with_graphic": c2,
        "Timeline - Timeline_bmap": c3,
        "Timeline - Timeline_sankey": c4,
        "Timeline - Timeline_bar_reversal": c5,
        "Timeline - Timeline_bar": c6,
        "Timeline - Timeline_multi_axis": c7,
        "Timeline - Timeline_map": c8
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/12 21:11
@File: logout.py
@IDE: pycharm
@Description:
    登出模块
"""
import shutil
import sys

import click
from rich.text import Text

from swanlab.env import get_save_dir
from swanlab.log import swanlog
from swanlab.package import has_api_key


@click.command()
def logout():
    """Logout to the swanlab cloud."""
    command = Text("swanlab login", "bold")
    if has_api_key():
        # 如果已经是登录状态，那么则询问用户是否确认，如果确认则删除token文件夹

        swanlog.info("Are you sure you want to logout? (y/N): ", end='')
        confirm = input()
        if confirm.lower() == "y":
            try:
                shutil.rmtree(get_save_dir())
                return swanlog.info(" Logout successfully. You can use `", command, "` to login again.", sep="")
            except Exception as e:
                return swanlog.info("Logout failed. Reason:" + str(e))
        else:
            return swanlog.info("Logout canceled.")
    # 如果还未登录，则不做任何处理，并告知用户如何登录
    swanlog.info("You are not logged in. If you want to login in, please use `", command, "` to login.", sep="")
    return sys.exit(1)



"""
// @author: ComPleHN
// @file: effectscatter.vue
// @time: 2025/5/27 15:30
// @description: 本文件是对于echarts的 涟漪散点图 图表的测试
"""
# ---------------------------------------------- Effectscatter - Effectscatter_symbol ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import EffectScatter
from pyecharts.faker import Faker
from pyecharts.globals import SymbolType

c1 = (
    EffectScatter()
    .add_xaxis(Faker.choose())
    .add_yaxis("", Faker.values(), symbol=SymbolType.ARROW)
    .set_global_opts(title_opts=opts.TitleOpts(title="EffectScatter-不同Symbol"))
)

# ---------------------------------------------- Effectscatter - Effectscatter_base ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import EffectScatter
from pyecharts.faker import Faker

c2 = (
    EffectScatter()
    .add_xaxis(Faker.choose())
    .add_yaxis("", Faker.values())
    .set_global_opts(title_opts=opts.TitleOpts(title="EffectScatter-基本示例"))
)

# ---------------------------------------------- Effectscatter - Effectscatter_splitline ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import EffectScatter
from pyecharts.faker import Faker

c3 = (
    EffectScatter()
    .add_xaxis(Faker.choose())
    .add_yaxis("", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="EffectScatter-显示分割线"),
        xaxis_opts=opts.AxisOpts(splitline_opts=opts.SplitLineOpts(is_show=True)),
        yaxis_opts=opts.AxisOpts(splitline_opts=opts.SplitLineOpts(is_show=True)),
    )
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="effect_scatter",
    public=True,
)

swanlab.log(
    {
        "Effectscatter - Effectscatter_symbol": c1,
        "Effectscatter - Effectscatter_base": c2,
        "Effectscatter - Effectscatter_splitline": c3
    }
)



"""
@author: cunyue
@file: __init__.py.py
@time: 2024/12/3 20:14
@description: 硬件信息采集
"""

from typing import Callable, List, Any, Optional, Tuple

from .cpu import get_cpu_info
from .dcu.hygon import get_hygon_dcu_info
from .disk import get_disk_info
from .gpu.metax import get_metax_gpu_info
from .gpu.moorethreads import get_moorethreads_gpu_info
from .gpu.nvidia import get_nvidia_gpu_info
from .memory import get_memory_size
from .mlu.cambricon import get_cambricon_mlu_info
from .network import get_network_info
from .npu.ascend import get_ascend_npu_info
from .soc.apple import get_apple_chip_info
from .type import HardwareFuncResult, HardwareCollector, HardwareInfo
from .utils import is_system_key
from .xpu.kunlunxin import get_kunlunxin_xpu_info

__all__ = ["get_hardware_info", "HardwareCollector", "HardwareInfo", "is_system_key"]


def get_hardware_info() -> Tuple[Optional[Any], List[HardwareCollector]]:
    """
    采集硬件信息，包括CPU、GPU、内存、硬盘等
    """
    monitor_funcs = []
    # 我们希望计算芯片的信息放在最前面，前端展示用
    nvidia = dec_hardware_func(get_nvidia_gpu_info, monitor_funcs)
    moorethreads = dec_hardware_func(get_moorethreads_gpu_info, monitor_funcs)
    ascend = dec_hardware_func(get_ascend_npu_info, monitor_funcs)
    cambricon = dec_hardware_func(get_cambricon_mlu_info, monitor_funcs)
    apple = dec_hardware_func(get_apple_chip_info, monitor_funcs)
    kunlunxin = dec_hardware_func(get_kunlunxin_xpu_info, monitor_funcs)
    metax = dec_hardware_func(get_metax_gpu_info, monitor_funcs)
    hygon = dec_hardware_func(get_hygon_dcu_info, monitor_funcs)
    c = dec_hardware_func(get_cpu_info, monitor_funcs)
    m = dec_hardware_func(get_memory_size, monitor_funcs)
    d = dec_hardware_func(get_disk_info, monitor_funcs)
    n = dec_hardware_func(get_network_info, monitor_funcs)

    info = {
        "memory": m,
        "cpu": c,
        "disk": d,
        "network": n,
        "gpu": {},
        "npu": {},
        "mlu": {},
        "xpu": {},
        "soc": {},
        "dcu": {},
    }
    if nvidia is not None:
        info["gpu"]["nvidia"] = nvidia
    if moorethreads is not None:
        info["gpu"]["moorethreads"] = moorethreads
    if metax is not None:
        info["gpu"]["metax"] = metax
    if ascend is not None:
        info["npu"]["ascend"] = ascend
    if apple is not None:
        info["soc"]["apple"] = apple
    if cambricon is not None:
        info["mlu"]["cambricon"] = cambricon
    if kunlunxin is not None:
        info["xpu"]["kunlunxin"] = kunlunxin
    if hygon is not None:
        info["dcu"]["hygon"] = hygon
    return filter_none(info, fallback={}), monitor_funcs


def dec_hardware_func(
    func: Callable[[], HardwareFuncResult],
    monitor_funcs: List[HardwareCollector],
) -> Optional[Any]:
    """
    装饰器，用于记录硬件信息采集函数
    """
    x, y = func()
    if y:
        monitor_funcs.append(y)
    return x


def filter_none(data, fallback=None):
    """
    过滤掉字典中值为None的键值对，只对字典有效
    """
    if isinstance(data, dict):
        data = {k: v for k, v in data.items() if v is not None and v != {}}  # 过滤掉空字典
        if all(v is None for v in data.values()):
            return fallback
    return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/13 01:11
@File: pytest_login.py
@IDE: pycharm
@Description:
    测试命令登录
"""
import os

import nanoid
import pytest
from click.testing import CliRunner

import tutils as T
from swanlab.cli.main import cli
from swanlab.env import SwanLabEnv
from swanlab.error import ValidationError, APIKeyFormatError
from swanlab.package import get_key


# noinspection PyTypeChecker
@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_login_ok():
    runner = CliRunner()
    result = runner.invoke(cli, ["login", "--api-key", T.API_KEY])
    assert result.exit_code == 0
    assert get_key() == T.API_KEY


# noinspection PyTypeChecker
@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_login_fail():
    runner = CliRunner()
    result = runner.invoke(cli, ["login", "--api-key", nanoid.generate(alphabet='12345', size=21)])
    assert result.exit_code == 1
    assert isinstance(result.exception, ValidationError)
    result = runner.invoke(cli, ["login", "--api-key", "wrong-key"])
    assert result.exit_code == 1
    assert isinstance(result.exception, APIKeyFormatError)


# noinspection PyTypeChecker
@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_login_host():
    """
    测试登录时指定host
    """
    runner = CliRunner()
    del os.environ[SwanLabEnv.API_HOST.value]  # 删除环境变量
    del os.environ[SwanLabEnv.WEB_HOST.value]  # 删除环境变量
    result = runner.invoke(cli, ["login", "--api-key", T.API_KEY, "--host", T.API_HOST.rstrip("/api")])
    assert result.exit_code == 0
    del os.environ[SwanLabEnv.API_HOST.value]  # 删除环境变量
    del os.environ[SwanLabEnv.WEB_HOST.value]  # 删除环境变量
    result = runner.invoke(cli, ["login", "--api-key", T.API_KEY, "--host", "http://wrong-host"])
    assert result.exit_code == 2



"""
@author: cunyue
@file: key.py
@time: 2025/7/2 11:12
@description: 每一个指标都需要一个唯一的key来标识，我们将它实现为一个对象
"""

import json
import math
from typing import Optional, Tuple

from swanlab.data.modules import DataWrapper, Line
from swanlab.env import create_time
from swanlab.log import swanlog
from swanlab.toolkit import (
    MetricInfo,
    ColumnInfo,
    MetricErrorInfo,
    ColumnClass,
    SectionType,
    ColumnConfig,
    ParseErrorInfo,
    ChartType,
)


class SwanLabKey:
    """
    运行时每个tag的配置，用于记录一些信息
    包括每个tag专属的图表配置
    """

    # 每__slice_size个tag数据保存为一个文件
    __slice_size = 1000

    def __init__(
        self,
        key: str,
        media_dir: str,
        log_dir: str,
    ) -> None:
        self.key = key
        # 当前 key 包含的 step
        self.steps = set()
        self.column_info: Optional[ColumnInfo] = None
        self._media_dir = media_dir
        self._log_dir = log_dir
        # 当前数据概要总结
        self._summary = {}
        # 当前key的数据集合
        self._collection = self.__new_metric_collection()

    @property
    def sum(self):
        """当前tag的数据总数"""
        return len(self.steps)

    @property
    def chart_created(self):
        """判断当前tag是否已经创建了图表"""
        return self.column_info is not None

    @property
    def is_chart_valid(self) -> bool:
        """判断当前tag对应的自动创建图表是否成功"""
        return self.column_info.error is None

    def add(self, data: DataWrapper) -> MetricInfo:
        """添加一个数据，在内部完成数据类型转换
        如果转换失败，打印警告并退出
        并且添加数据，当前的数据保存是直接保存，后面会改成缓存形式
        进入此函数之前column_info必须已经创建

        Parameters
        ----------
        data : DataType
            待添加的数据

        Returns
        -------
        metric_info
            添加数据的信息，如果ok为False，则返回None
        """
        # 1. 不允许在列创建前调用此方法
        assert self.column_info is not None, "Column info is None, please create column info first"
        result = data.parse()
        # 2. 解析失败则报错
        if data.error is not None:
            swanlog.warning(
                f"Log failed. Reason: Data on key '{self.key}' (step {result.step}) cannot be converted ."
                f"It should be {data.error.expected}, but it is {data.error.got}, please check the data type."
            )
            return MetricErrorInfo(column_info=self.column_info, error=data.error)
        # 3. 前后图表类型不一致则报错
        now_chart = result.chart
        expected_chart = self.column_info.chart_type
        if now_chart != expected_chart:
            swanlog.error(
                f"Data type error, key: {self.key}, "
                f"data type: {now_chart.value.column_type}, expected: {expected_chart.value.column_type}."
            )
            return MetricErrorInfo(
                column_info=self.column_info,
                error=ParseErrorInfo(
                    expected=expected_chart.value.column_type,
                    got=now_chart.value.column_type,
                    chart=result.chart,
                ),
            )
        # 4. 更新 summary 并添加数据
        # 如果为Line且为NaN或者INF，不更新summary
        r = result.strings or result.float
        if not data.type == Line or r not in [Line.nan, Line.inf]:
            if self._summary.get("max") is None or r > self._summary["max"]:
                self._summary["max"] = r
                self._summary["max_step"] = result.step
            if self._summary.get("min") is None or r < self._summary["min"]:
                self._summary["min"] = r
                self._summary["min_step"] = result.step
        self._summary["num"] = self._summary.get("num", 0) + 1
        self.steps.add(result.step)
        swanlog.debug(f"Add data, key: {self.key}, step: {result.step}, data: {r}")
        if len(self._collection["data"]) >= self.__slice_size:
            self._collection = self.__new_metric_collection()

        new_data = self.__new_metric(result.step, r, more=result.more)
        self._collection["data"].append(new_data)
        epoch = len(self.steps)
        mu = math.ceil(epoch / self.__slice_size)
        return MetricInfo(
            column_info=self.column_info,
            metric=json.loads(json.dumps(new_data)),
            metric_summary=json.loads(json.dumps(self._summary)),
            metric_epoch=epoch,
            metric_step=result.step,
            metric_buffers=result.buffers,
            metric_file_name=str(mu * self.__slice_size) + ".log",
            swanlab_logdir=self._log_dir,
            swanlab_media_dir=self._media_dir if result.buffers else None,
        )

    def create_column(
        self,
        key: str,
        name: Optional[str],
        column_class: ColumnClass,
        column_config: Optional[ColumnConfig],
        section_type: SectionType,
        data: DataWrapper,
        num: int,
    ) -> ColumnInfo:
        """
        创建列信息，对当前key的基本信息做一个记录
        :param key: str, key名称
        :param name: str, key的实际名称
        :param column_class: str, key的类型
        :param column_config: ColumnConfig, key的配置
        :param section_type: str, key的组类型
        :param data: DataType, 数据
        :param num: 创建此列之前的列数量
        """
        assert self.column_info is None, "Cannot create column info after creating it"
        result = data.parse()
        # SYSTEM PINNED HIDDEN 三个类型的 section 不应该传递名称
        # PUBLIC 可选是否传递名称，如果 key 包含斜杠，则使用斜杠前的部分作为section的名称
        # CUSTOM 时如果 key 包含斜杠，则使用斜杠前的部分作为section的名称，并且将 section_type 设置为 PUBLIC
        if section_type in ["PUBLIC", "CUSTOM"]:
            split_key = key.split("/")
            if len(split_key) > 1 and split_key[0]:
                # 如果key包含斜杠，则使用斜杠前的部分作为section的名称
                result.section = split_key[0]
                section_type: SectionType = "PUBLIC"
        else:
            result.section = None
        column_info = ColumnInfo(
            key=key,
            kid=str(num),
            name=name,
            cls=column_class,
            config=column_config,
            chart_type=result.chart,
            section_name=result.section,
            section_type=section_type,
            section_sort=None,
            chart_reference=result.reference,
            error=data.error,
        )
        self.column_info = column_info
        return column_info

    @staticmethod
    def __new_metric(index, data, more: dict = None) -> dict:
        """创建一个新的data数据，实际上是一个字典，包含一些默认信息

        Parameters
        ----------
        index : int
            步数
        data : DataType
            数据
        more : dict, optional
            更多的数据，如果有的话
        """
        if more is None:
            return {
                "index": int(index),
                "data": data,
                "create_time": create_time(),
            }
        else:
            return {
                "index": int(index),
                "data": data,
                "create_time": create_time(),
                "more": more,
            }

    @staticmethod
    def __new_metric_collection() -> dict:
        """创建一个新的key data数据集合

        Returns
        -------
        dict
            返回一个新的data数据集合
        """
        time = create_time()
        return {
            "create_time": time,
            "update_time": time,
            "data": [],
        }

    @classmethod
    def mock_from_remote(
        cls,
        key: str,
        column_type: str,
        column_class: ColumnClass,
        error: Optional[dict],
        media_dir: str,
        log_dir: str,
        kid: int,
        step: Optional[int],
        name: str = None,
    ) -> Tuple['SwanLabKey', ColumnInfo]:
        """
        从远程数据创建一个SwanLabKey对象，主要用于 Resume 时的图表数据标记
        mock的 column 不会被用于上传数据，只是用于标记和记录错误信息
        此对象不会用于上传数据，只是用于标记和记录错误信息
        :param key: str, key名称
        :param column_type: str, 列类型
        :param column_class: ColumnClass, 列的类型
        :param error: 列错误信息
        :param media_dir: str, 媒体目录
        :param log_dir: str, 指标目录
        :param kid: int, 代表这个列是第几个列，通常是从0开始的整数
        :param step: int, 云端最新步数，如果不传则默认为None
        :param name: str, 列的实际名称, 如果不传则默认为key
        """
        if name is None:
            name = key
        # 1. 创建对象
        key_obj = cls(key, media_dir, log_dir)
        # 2. 生成一个 ColumnInfo 对象，此 column 对象不会被用于上传
        if column_type == "FLOAT":
            chart = ChartType.LINE
        else:
            chart = getattr(ChartType, column_type, None)
        if chart is None:
            raise RuntimeError(
                f"Unknown chart type: {column_type}, maybe you need to update swanlab: pip install -U swanlab"
            )

        if error is not None:
            expected = error.get("excepted")
            got = error.get("data_class")
            if expected is None or got is None:
                raise RuntimeError(
                    f"Invalid error format: {error}, expected and got must be provided. "
                    f"Maybe you need to update swanlab: pip install -U swanlab"
                )
            error = ParseErrorInfo(expected=expected, got=got, chart=chart)
        # 3. 根据 column_class 和 chart 适配 section_type
        if column_class == "SYSTEM":
            section_type: SectionType = "SYSTEM"
        else:
            if chart is ChartType.ECHARTS:
                section_type = "CUSTOM"
            else:
                section_type = "PUBLIC"
        # 4. 创建 ColumnInfo 对象
        column_info = ColumnInfo(
            key,
            str(kid),
            name,
            column_class,
            chart,
            chart_reference="STEP",
            error=error,
            section_name=None,
            section_type=section_type,
        )
        key_obj.column_info = column_info
        # 5. 设置当前步数，resume 后不允许设置历史步数，所以需要覆盖
        if step is not None:
            for i in range(step + 1):
                key_obj.steps.add(i)
        return key_obj, column_info



from tutils import open_dev_mode
import swanlab

swanlab.login(open_dev_mode())

from fastai.vision.all import *
from swanlab.integration.fastai import SwanLabCallback


# 加载数据
path = untar_data(URLs.PETS)
dls = ImageDataLoaders.from_name_re(
    path, get_image_files(path / "images"), pat=r"([^/]+)_\d+.jpg$", item_tfms=Resize(224)
)

# 定义模型
learn = vision_learner(dls, resnet34, metrics=error_rate)

# 添加SwanLabCallback
learn.fit_one_cycle(
    5,
    cbs=SwanLabCallback(
        project="fastai-swanlab-integration-test",
        experiment_name="super-test",
        description="Test fastai integration with swanlab",
        logdir="./logs",
    ),
)



"""SwanLab Point Cloud Module

This module provides classes for handling point cloud data in various formats.

Classes:
    PointCloud: Class for handling point cloud data with different formats (XYZ, XYZC, XYZRGB)

Examples:
    Create from XYZ coordinates:
    >>> import numpy as np
    >>> points = np.random.rand(100, 3)  # 100 points with XYZ coordinates
    >>> pc = PointCloud.from_xyz(points)  # Default green color
    >>> pc = PointCloud.from_xyz(points, caption="My Points")

    Create from XYZC format (XYZ + category):
    >>> points_with_category = np.zeros((100, 4))  # XYZ + category
    >>> points_with_category[:, :3] = points  # XYZ coordinates
    >>> points_with_category[:, 3] = categories  # Category labels (0,1,2...)
    >>> pc = PointCloud.from_xyzc(points_with_category)  # Auto color mapping

    Create from XYZRGB format:
    >>> points_with_color = np.zeros((100, 6))  # XYZRGB format
    >>> points_with_color[:, :3] = points  # XYZ coordinates
    >>> points_with_color[:, 3:] = colors  # RGB values (0-255)
    >>> pc = PointCloud.from_xyzrgb(points_with_color)

    Load from SwanLab pts.json file:
    >>> from pathlib import Path
    >>> pc = PointCloud.from_swanlab_pts_json_file(
    ...     Path("./file/points.swanlab.pts.json"),
    ...     caption="Loaded Points"
    ... )
"""

import json
from dataclasses import dataclass, field
from pathlib import Path
from typing import Dict, List, Optional, Tuple, TypedDict

from swanlab.data.namer import hex_to_rgb, light_colors
from swanlab.toolkit import DataSuite as D, MediaBuffer, MediaType

try:
    import numpy as np

    ndarray = np.ndarray
except ImportError:
    np = None
    ndarray = None


class Box(TypedDict):
    """
    Represents a 3D bounding box with associated metadata.

    Attributes:
        color (Tuple[int, int, int]): The RGB (0..255) color of the bounding box (e.g., [255, 255, 0] for yellow).
        corners (List[List[float, float, float]]): A list of 8 corner points defining the 3D bounding box.
            Each corner is a list of three floats representing the (x, y, z) coordinates.

            The order of the corners is as follows (assuming a right-handed coordinate system):

            *   0: Bottom-Left-Back
            *   1: Bottom-Right-Back
            *   2: Bottom-Right-Front
            *   3: Bottom-Left-Front
            *   4: Top-Left-Back
            *   5: Top-Right-Back
            *   6: Top-Right-Front
            *   7: Top-Left-Front

        label (str): A string representing the object category or class label (e.g., "match").
        score (float): A confidence score representing the certainty of the bounding box's object detection.
    """

    color: Tuple[int, int, int]
    corners: List[Tuple[float, float, float]]
    label: str
    score: Optional[float]


@dataclass()
class PointCloud(MediaType):
    """Point cloud data representation.

    Supports various formats including XYZ, XYZC (XYZ + category), and XYZRGB.
    Internally stores data in XYZRGB format.

    Attributes:
        points: Point cloud data array with shape (N, 6) containing XYZRGB values
        caption: Optional description text

    Examples:
        >>> import numpy as np
        >>> points = np.random.rand(100, 3)  # XYZ format
        >>> pc = PointCloud.from_xyz(points)  # Create with default green color
        >>> pc = PointCloud.from_xyz(points, caption="My Points")  # With caption
    """

    points: ndarray  # format xyzrgb
    boxes: List[Box] = field(default_factory=list)
    step: Optional[int] = None
    caption: Optional[str] = None
    key: Optional[str] = None
    _VERSION: str = "0.1"

    def __post_init__(self):
        """Validate input data after initialization"""
        self._check_numpy()

        if self.points.ndim != 2 or self.points.shape[1] != 6:
            raise ValueError("PointCloud(points) points must have shape (N, 6)")

    @staticmethod
    def _check_numpy() -> None:
        if np is None:
            raise ImportError("Numpy is required for PointCloud class. Please install it with: pip install numpy")

    @classmethod
    def from_xyz(cls, points: ndarray, *, caption: Optional[str] = None, **kwargs) -> "PointCloud":
        """Create PointCloud from XYZ coordinates.

        Args:
            points: numpy array with shape (N, 3) containing XYZ coordinates
            caption: Optional description text

        Returns:
            PointCloud object with default green color

        Examples:
            >>> points = np.random.rand(100, 3)
            >>> pc = PointCloud.from_xyz(points)  # Default green
            >>> pc = PointCloud.from_xyz(points, caption="Green Points")
        """
        if points.ndim != 2 or points.shape[1] != 3:
            raise ValueError("XYZ array must have shape (N, 3)")

        default_color = np.array([0, 255, 0])  # green

        xyzrgb = np.zeros((points.shape[0], 6))
        xyzrgb[:, :3] = points  # copy XYZ coordinates
        xyzrgb[:, 3:] = default_color  # set default RGB (0, 255, 0) to green

        return cls(xyzrgb, caption=caption, **kwargs)

    @classmethod
    def from_xyzc(cls, points: ndarray, *, caption: Optional[str] = None, **kwargs) -> "PointCloud":
        """Create PointCloud from XYZC format (XYZ coordinates + category).

        Args:
            points: numpy array with shape (N, 4) containing XYZC values
                   where C is category index (integer)
            caption: Optional description text

        Returns:
            PointCloud object with colors mapped from categories

        Examples:
            >>> points = np.zeros((100, 4))
            >>> points[:, :3] = coordinates  # XYZ coordinates
            >>> points[:, 3] = categories    # Category labels (0,1,2...)
            >>> pc = PointCloud.from_xyzc(points)
            >>> pc = PointCloud.from_xyzc(points, caption="Segmented Points")
        """
        if points.ndim != 2 or points.shape[1] != 4:
            raise ValueError("XYZC array must have shape (N, 4)")

        # For xyzc format, map categories to predefined colors
        categories = points[:, 3].astype(int)
        colors = np.array([hex_to_rgb(c) for c in light_colors])

        xyzrgb = np.zeros((points.shape[0], 6))
        xyzrgb[:, :3] = points[:, :3]  # copy XYZ coordinates
        xyzrgb[:, 3:] = colors[categories]

        return cls(xyzrgb, caption=caption, **kwargs)

    @classmethod
    def from_xyzrgb(cls, points: ndarray, *, caption: Optional[str] = None, **kwargs) -> "PointCloud":
        """Create PointCloud from XYZRGB format.

        Args:
            points: numpy array with shape (N, 6) containing XYZRGB values
            caption: Optional description text

        Returns:
            PointCloud object

        Examples:
            >>> points = np.zeros((100, 6))
            >>> points[:, :3] = coordinates  # XYZ coordinates
            >>> points[:, 3:] = colors       # RGB values (0-255)
            >>> pc = PointCloud.from_xyzrgb(points)
            >>> pc = PointCloud.from_xyzrgb(points, step=1, caption="Colored Points")
        """
        if points.ndim != 2 or points.shape[1] != 6:
            raise ValueError("XYZRGB array must have shape (N, 6)")

        return cls(points, caption=caption, **kwargs)

    @classmethod
    def from_swanlab_pts(cls, data: Dict, *, caption: Optional[str] = None, **kwargs) -> "PointCloud":
        """Create PointCloud from SwanLab pts data dictionary.

        Args:
            data: A dictionary containing 'points' (required) and optionally 'boxes'.
                  'points' can be a list of lists (XYZ, XYZC, or XYZRGB) or a NumPy array.
                  'boxes' is an optional list of dictionaries, each representing a bounding box.
            caption: Optional description text

        Returns:
            PointCloud object
        """
        if not isinstance(data, dict) or "points" not in data:
            raise ValueError("Invalid data format")

        points = data["points"]
        if isinstance(points, list):
            points = np.array(points)  # Convert list to NumPy array
        elif not isinstance(points, ndarray):
            raise TypeError("data['points'] must be a list or a NumPy array")

        if points.ndim != 2:
            raise ValueError("data['points'] must be a 2D array")

        handler = {3: cls.from_xyz, 4: cls.from_xyzc, 6: cls.from_xyzrgb}

        try:
            pc = handler[points.shape[1]](points, caption=caption, **kwargs)
        except KeyError as err:
            raise ValueError("data['points'] must have shape (N, 3), (N, 4), or (N, 6)") from err

        # Add boxes
        boxes_data = data.get("boxes")
        if boxes_data is None:
            return pc

        if not isinstance(boxes_data, list):
            raise TypeError("data['boxes'] must be a list")

        for box_data in boxes_data:
            try:
                box: Box = {
                    "color": tuple(box_data["color"]),
                    "corners": [tuple(p) for p in box_data["corners"]],
                    "label": box_data["label"],
                }
                if "score" in box_data:
                    box["score"] = box_data["score"]
                pc.append_box(box)
            except (KeyError, TypeError) as err:
                raise ValueError(f"Invalid box format: {err}") from err

        return pc

    @classmethod
    def from_swanlab_pts_json_file(cls, path: Path, *, caption: Optional[str] = None, **kwargs) -> "PointCloud":
        """Create PointCloud from SwanLab pts.json file.

        Args:
            path: Path to the .swanlab.pts.json file
            caption: Optional description text

        Returns:
            PointCloud object

        Examples:
            >>> pc = PointCloud.from_swanlab_pts_json_file(
            ...     Path("points.swanlab.pts.json"),
            ...     caption="Loaded Points"
            ... )
        """
        if not path.exists():
            raise FileNotFoundError(f"File not found: {path}")

        if not path.is_file():
            raise ValueError(f"Path is not a file: {path}")

        try:
            with open(path) as f:
                points_list = json.load(f)

            return cls.from_swanlab_pts(points_list, caption=caption, **kwargs)

        except json.JSONDecodeError as e:
            raise ValueError(f"Invalid JSON format in file: {path}") from e
        except Exception as e:
            raise ValueError(f"Error reading file {path}: {str(e)}") from e

    def append_box(self, box: Box):
        """Append a bounding box to the point cloud.

        Args:
            box: The bounding box to append.
        """
        self.boxes.append(box)

    def extend_boxes(self, boxes: List[Box]):
        """Extend the list of bounding boxes with a list of boxes.

        Args:
            boxes: The list of bounding boxes to extend.
        """
        self.boxes.extend(boxes)

    # ---------------------------------- override ----------------------------------

    def parse(self) -> Tuple[str, MediaBuffer]:
        """Convert point cloud to buffer for transmission.

        Returns:
            Tuple containing:
            - File name with format: pointcloud-step{step}-{hash}.swanlab.pts.json
            - MediaBuffer containing the point cloud data
        """
        buffer = MediaBuffer()
        points_list = self.points.tolist()
        swanlab_pts = {
            "version": self._VERSION,
            "points": points_list,
            "boxes": self.boxes,
        }
        json_str = json.dumps(swanlab_pts)
        buffer.write(json_str.encode())

        hash_name = D.get_hash_by_ndarray(self.points)[:16]
        save_name = f"pointcloud-step{self.step}-{hash_name}.swanlab.pts.json"

        return save_name, buffer

    def get_chart(self) -> MediaType.Chart:
        """Return chart type for visualization"""
        return MediaType.Chart.OBJECT3D

    def get_section(self) -> str:
        """Return section name for organization"""
        return "PointsCloud"

    def get_more(self) -> Optional[Dict[str, str]]:
        """Return additional information (caption)"""
        return {"caption": self.caption} if self.caption else None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/13 01:28
@File: pytest_logout.py
@IDE: pycharm
@Description:
    测试登出模块
"""
import pytest
from click.testing import CliRunner

import tutils as T
from swanlab.cli import cli


# noinspection PyTypeChecker
@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_logout_ok(monkeypatch):
    runner = CliRunner()
    # 先登录
    runner.invoke(cli, ["login", "--api-key", T.API_KEY])
    monkeypatch.setattr("builtins.input", lambda: "y")
    result = runner.invoke(cli, ["logout"])
    assert result.exit_code == 0


# noinspection PyTypeChecker
@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_logout_cancel(monkeypatch):
    runner = CliRunner()
    # 先登录
    runner.invoke(cli, ["login", "--api-key", T.API_KEY])
    monkeypatch.setattr("builtins.input", lambda: "n")
    result = runner.invoke(cli, ["logout"])
    assert result.exit_code == 0


# noinspection PyTypeChecker
@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_logout_no_login():
    runner = CliRunner()
    result = runner.invoke(cli, ["logout"])
    assert result.exit_code == 1
    assert "You are not logged in." in result.output



"""
@author: cunyue
@file: session.py
@time: 2025/9/9 15:10
@description: 创建会话
"""

import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from swanlab.package import get_package_version


def create_session() -> requests.Session:
    """
    创建一个带重试机制的会话
    :return: requests.Session
    """
    session = requests.Session()
    retry = Retry(
        total=5,
        backoff_factor=0.5,
        status_forcelist=[429, 500, 502, 503, 504],
        allowed_methods=frozenset(["GET", "POST", "PUT", "DELETE", "PATCH"]),
    )
    adapter = HTTPAdapter(max_retries=retry)
    session.mount("https://", adapter)
    session.mount("http://", adapter)
    session.headers["swanlab-sdk"] = get_package_version()
    return session



"""
Docs:https://docs.swanlab.cn/guide_cloud/integration/integration-pytorch-torchtune.html
"""

from torchtune.utils._distributed import get_world_size_and_rank
from torchtune.utils.metric_logging import MetricLoggerInterface
from typing import Mapping, Optional, Union, Any, Dict, List

from numpy import ndarray
from omegaconf import DictConfig, OmegaConf
from torch import Tensor

Scalar = Union[Tensor, ndarray, int, float]


class SwanLabLogger(MetricLoggerInterface):
    """
    Example:
        In torchtune config:
        >>> # Logging
        >>> metric_logger:
        >>>     _component_: swanlab.integration.huggingface.SwanLabLogger
        >>>     project: "gemma-lora-finetune"
        >>>     experiment_name: "gemma-2b"
        >>>     log_dir: ${output_dir}

    Raises:
        ImportError: If ``swanlab`` package is not installed.

    Note:
        This logger requires the swanlab package to be installed.
        You can install it with `pip install swanlab`.
        In order to use the logger, you need to login to your SwanLab account.
        You can do this by running `swanlab login` in your terminal.
    """

    def __init__(
        self,
        project: str = "torchtune",
        workspace: Optional[str] = None,
        experiment_name: Optional[str] = None,
        description: Optional[str] = None,
        mode: Optional[str] = None,
        log_dir: Optional[str] = None,
        tags: Optional[List[str]] = None,
        **kwargs,
    ):
        try:
            import swanlab
        except ImportError as e:
            raise ImportError(
                "``swanlab`` package not found. Please install swanlab using `pip install swanlab` to use SwanLabLogger."
            ) from e
        self._swanlab = swanlab
        swanlab.config["FRAMEWORK"] = "torchtune"

        tags = tags or []
        tags.append("torchtune") if "torchtune" not in tags else None

        # Use dir if specified, otherwise use log_dir.
        self.log_dir = kwargs.pop("dir", log_dir)

        _, self.rank = get_world_size_and_rank()

        if self._swanlab.get_run() is None and self.rank == 0:
            run = self._swanlab.init(
                project=project,
                workspace=workspace,
                name=experiment_name,
                description=description,
                mode=mode,
                logdir=self.log_dir,
                tags=tags,
                **kwargs,
            )

        self.config_allow_val_change = kwargs.get("allow_val_change", False)

    def update_config(self, config: Dict[str, Any]):
        self._swanlab.config.update(config)

    def log_config(self, config: DictConfig) -> None:
        if self._swanlab.get_run():
            resolved = OmegaConf.to_container(config, resolve=True)
            self._swanlab.config.update(resolved, allow_val_change=self.config_allow_val_change)

    def log(self, name: str, data: Scalar, step: int) -> None:
        if self._swanlab.get_run():
            self._swanlab.log({name: data}, step=step)

    def log_dict(self, payload: Mapping[str, Scalar], step: int) -> None:
        if self._swanlab.get_run():
            self._swanlab.log({**payload}, step=step)

    def __del__(self) -> None:
        if self._swanlab.get_run():
            self._swanlab.finish()

    def close(self) -> None:
        if self._swanlab.get_run():
            self._swanlab.finish()



"""
@author: cunyue
@file: 3d_object.py
@time: 2025/3/13 13:30
@description: 测试上传3d对象

NOTE 你需要下载下面的文件放到当前文件目录的assets文件夹下，才能运行这个测试
- 3D点云数据（带标注）: https://drive.google.com/file/d/1mFill-BXw3cirPHwIHndb1wNX4pWvSXb/view
"""

import json
import os

# noinspection PyPackageRequirements
import numpy as np

import swanlab

current_path = os.path.dirname(os.path.abspath(__file__))

# ---------------------------------- 点云数据/swanlab点云文件路径 ----------------------------------

cloud_point_path = os.path.join(current_path, "assets/data.swanlab.pts.json")

with open(cloud_point_path, "r") as f:
    cloud_point = json.load(f)


# ---------------------------------- 从代码生成的点云矩阵 ----------------------------------


def mock_object_3d_array() -> np.ndarray:
    return np.random.rand(100, 3)


# ---------------------------------- 实验 ----------------------------------


swanlab.init(project="3d-object-project", config={"epochs": 10}, mode="cloud")


for epoch in range(2, swanlab.config.epochs):
    swanlab.log(
        {
            "code/random": [swanlab.Object3D(mock_object_3d_array()) for _ in range(epoch)],
            "epoch": epoch,
            "code/point_cloud": swanlab.Object3D.from_point_data(
                points=cloud_point['points'], boxes=cloud_point['boxes'], caption="Point Cloud from cloud"
            ),
            "file/point_cloud": swanlab.Object3D(cloud_point_path, caption="Point Cloud from file"),
        }
    )



import xgboost as xgb
from sklearn.datasets import load_breast_cancer
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score, classification_report
import swanlab
from swanlab.integration.xgboost import SwanLabCallback

# 初始化swanlab
swanlab.init(project="xgboost-breast-cancer", config={
    "learning_rate": 0.1,
    "max_depth": 3,
    "subsample": 0.8,
    "colsample_bytree": 0.8,
    "num_round": 100
})

# 加载数据集
data = load_breast_cancer()
X = data.data
y = data.target

# 将数据集分为训练集和测试集
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

# 转换为DMatrix格式，这是XGBoost的内部数据格式
dtrain = xgb.DMatrix(X_train, label=y_train)
dtest = xgb.DMatrix(X_test, label=y_test)

# 设置参数
params = {
    'objective': 'binary:logistic',  # 二分类任务
    'max_depth': 3,                  # 树的最大深度
    'eta': 0.1,                      # 学习率
    'subsample': 0.8,                # 样本采样比例
    'colsample_bytree': 0.8,         # 特征采样比例
    'eval_metric': 'logloss'         # 评估指标
}

# 训练模型
num_round = 100  # 迭代次数
bst = xgb.train(params, dtrain, num_round, evals=[(dtrain, 'train'), (dtest, 'test')], callbacks=[SwanLabCallback()])

# 进行预测
y_pred = bst.predict(dtest)
y_pred_binary = [round(value) for value in y_pred]  # 将概率转换为二分类结果

# 评估模型
accuracy = accuracy_score(y_test, y_pred_binary)
print(f"Accuracy: {accuracy:.4f}")

# 打印分类报告
print("Classification Report:")
print(classification_report(y_test, y_pred_binary, target_names=data.target_names))

# 保存模型
bst.save_model('xgboost_model.model')

# 结束swanlab会话
swanlab.finish()


"""
// @author: ComPleHN
// @file: tree.vue
// @time: 2025/5/29 12:20
// @description: 本文件是对于echarts的 树状图 图表的测试,本文件有注意事项请看本目录下 README.md
"""
# ---------------------------------------------- Tree - Tree_base ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Tree


data = [
    {
        "children": [
            {"name": "B"},
            {
                "children": [{"children": [{"name": "I"}], "name": "E"}, {"name": "F"}],
                "name": "C",
            },
            {
                "children": [
                    {"children": [{"name": "J"}, {"name": "K"}], "name": "G"},
                    {"name": "H"},
                ],
                "name": "D",
            },
        ],
        "name": "A",
    }
]
c1 = (
    Tree()
    .add("", data)
    .set_global_opts(title_opts=opts.TitleOpts(title="Tree-基本示例"))
)

# ---------------------------------------------- Tree - Tree_bottom_top ----------------------------------------------
import json

from pyecharts import options as opts
from pyecharts.charts import Tree

with open("../assets/echarts/tree/flare.json", "r", encoding="utf-8") as f:
    j = json.load(f)
c2 = (
    Tree()
    .add(
        "",
        [j],
        collapse_interval=2,
        orient="BT",
        label_opts=opts.LabelOpts(
            position="top",
            horizontal_align="right",
            vertical_align="middle",
            rotate=-90,
        ),
    )
    .set_global_opts(title_opts=opts.TitleOpts(title="Tree-下上方向"))
)

# ---------------------------------------------- Tree - Tree_right_left ----------------------------------------------
import json

from pyecharts import options as opts
from pyecharts.charts import Tree

with open("../assets/echarts/tree/flare.json", "r", encoding="utf-8") as f:
    j = json.load(f)
c3 = (
    Tree()
    .add("", [j], collapse_interval=2, orient="RL")
    .set_global_opts(title_opts=opts.TitleOpts(title="Tree-右左方向"))
)

# ---------------------------------------------- Tree - Tree_left_right ----------------------------------------------
import json

from pyecharts import options as opts
from pyecharts.charts import Tree


with open("../assets/echarts/tree/flare.json", "r", encoding="utf-8") as f:
    j = json.load(f)
c4 = (
    Tree()
    .add("", [j], collapse_interval=2)
    .set_global_opts(title_opts=opts.TitleOpts(title="Tree-左右方向"))
)

# ---------------------------------------------- Tree - Tree_top_bottom ----------------------------------------------
import json

from pyecharts import options as opts
from pyecharts.charts import Tree


with open("../assets/echarts/tree/flare.json", "r", encoding="utf-8") as f:
    j = json.load(f)
c5 = (
    Tree()
    .add(
        "",
        [j],
        collapse_interval=2,
        orient="TB",
        label_opts=opts.LabelOpts(
            position="top",
            horizontal_align="right",
            vertical_align="middle",
            rotate=-90,
        ),
    )
    .set_global_opts(title_opts=opts.TitleOpts(title="Tree-上下方向"))
)

# ---------------------------------------------- Tree - Radial_tree ----------------------------------------------
import asyncio
from aiohttp import TCPConnector, ClientSession

import pyecharts.options as opts
from pyecharts.charts import Tree

"""
Gallery 使用 pyecharts 1.1.0
参考地址: https://echarts.apache.org/examples/editor.html?c=tree-radial

目前无法实现的功能:

1、
"""


async def get_json_data(url: str) -> dict:
    async with ClientSession(connector=TCPConnector(ssl=False)) as session:
        async with session.get(url=url) as response:
            return await response.json()


# 获取官方的数据
data = asyncio.run(
    get_json_data(url="https://echarts.apache.org/examples/data/asset/data/flare.json")
)

c6 = (
    Tree()
    .add(
        series_name="",
        data=[data],
        pos_top="18%",
        pos_bottom="14%",
        layout="radial",
        symbol="emptyCircle",
        symbol_size=7,
    )
    .set_global_opts(
        tooltip_opts=opts.TooltipOpts(trigger="item", trigger_on="mousemove")
    )
)

# ---------------------------------------------- Tree - Tree_layout ----------------------------------------------
import json

from pyecharts import options as opts
from pyecharts.charts import Tree

with open("../assets/echarts/tree/flare.json", "r", encoding="utf-8") as f:
    j = json.load(f)
c7 = (
    Tree()
    .add("", [j], collapse_interval=2, layout="radial")
    .set_global_opts(title_opts=opts.TitleOpts(title="Tree-Layout"))
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="tree",
    public=True,
)

swanlab.log(
    {
        "Tree - Tree_base": c1,
        "Tree - Tree_bottom_top": c2,
        "Tree - Tree_right_left": c3,
        "Tree - Tree_left_right": c4,
        "Tree - Tree_top_bottom": c5,
        "Tree - Radial_tree": c6,
        "Tree - Tree_layout": c7
    }
)



import json

import swanlab
from swanlab.data.modules import Echarts


def test_echarts():
    """测试图表功能"""
    chart = swanlab.echarts.Bar().add_xaxis(["X", "Y"]).add_yaxis("数值", [10, 20])
    assert isinstance(chart, swanlab.echarts.Bar)
    c = Echarts(chart)
    assert isinstance(c, Echarts)
    filename, buffer = c.parse()
    # 返回文件名称
    assert isinstance(filename, str)
    assert filename.endswith(".json")
    assert buffer is not None
    # 可解析为字典
    chart_data = json.loads(buffer.getvalue().decode("utf-8"))
    assert isinstance(chart_data, dict)



"""
@author: cunyue
@file: utils.py
@time: 2024/12/9 16:33
@description: 硬件信息采集工具函数
"""

import random

import psutil

from .type import HardwareInfo, HardwareInfoList, HardwareConfig

ALPHABET = "abcdefghijklmnopqrstuvwxyz0123456789"


def random_index(length: int = 8) -> str:
    """
    随机生成八位字符串，用于标识图表的index
    """
    return "".join(random.choices(ALPHABET, k=length))


def generate_key(suffix: str) -> str:
    """
    生成key，用于标识系统列，避免与用户输入的key冲突
    """
    return "__swanlab__." + suffix


def is_system_key(key: str) -> bool:
    """
    判断key是否为系统key
    系统key以 __swanlab__. 开头
    """
    return key.startswith("__swanlab__.")


# CPU 使用率
CPU_PCT_KEY = generate_key("cpu.pct")
CPU_PCT_CONFIG = HardwareConfig(
    y_range=(0, 100),
    chart_name="CPU Utilization (%)",
).clone()

# CPU 核心使用率
CPU_INDEX_PER_KEY = generate_key("cpu.{idx}.pct")
CPU_INDEX_PER_CONFIG = HardwareConfig(
    y_range=(0, 100),
    chart_name="CPU Utilization (per core) (%)",
    chart_index=random_index(),
)
CPU_INDEX_PER_CONFIGS = [CPU_INDEX_PER_CONFIG.clone(metric_name=f"CPU {idx}") for idx in range(psutil.cpu_count())]

# CPU 线程数
CPU_THDS_KEY = generate_key("cpu.thds")
CPU_THDS_CONFIG = HardwareConfig(
    y_range=(0, None),
    chart_name="Process CPU Threads",
).clone()


class CpuBaseCollector:
    """
    cpu采集基类，为子类赋予cpu采集的能力
    """

    __per_cpu_configs = []

    @staticmethod
    def get_cpu_usage() -> HardwareInfo:
        """
        获取当前 CPU 使用率
        """
        return {
            "key": CPU_PCT_KEY,
            "name": "CPU Utilization (%)",
            "value": psutil.cpu_percent(interval=1),
            "config": CPU_PCT_CONFIG,
        }

    @staticmethod
    def get_per_cpu_usage() -> HardwareInfoList:
        """
        获取每个 CPU 核心的使用率
        """
        per_cpu_usages = psutil.cpu_percent(interval=1, percpu=True)
        result: HardwareInfoList = []
        for idx, value in enumerate(per_cpu_usages):
            info: HardwareInfo = {
                "key": CPU_INDEX_PER_KEY.format(idx=idx),
                "name": f"CPU {idx} Utilization (%)",
                "value": value,
                "config": CPU_INDEX_PER_CONFIGS[idx],
            }
            result.append(info)
        return result

    @staticmethod
    def get_cur_proc_thds_num(proc: psutil.Process) -> HardwareInfo:
        """
        获取当前进程的线程数
        """
        return {
            "key": CPU_THDS_KEY,
            "name": "Process CPU Threads",
            "value": proc.num_threads(),
            "config": CPU_THDS_CONFIG,
        }


# 内存使用率
MEM_PCT_KEY = generate_key("mem.pct")
MEM_PCT_CONFIG = HardwareConfig(
    y_range=(0, 100),
).clone()

# 进程内存使用情况
MEM_PROC_KEY = generate_key("mem.proc")
MEM_PROC_CONFIG = HardwareConfig(
    y_range=(0, None),
    chart_name="Process Memory In Use (non-swap) (MB)",
).clone()

# 进程内存使用率
MEM_PROC_PCT_KEY = generate_key("mem.proc.pct")
MEM_PROC_PCT_CONFIG = HardwareConfig(
    y_range=(0, 100),
    chart_name="Process Memory Utilization (%)",
).clone()

# 进程内存可用情况
PROC_MEM_AVAIL_KEY = generate_key("mem.proc.avail")
PROC_MEM_AVAIL_CONFIG = HardwareConfig(
    y_range=(0, None),
    chart_name="Process Memory Available (non-swap) (MB)",
).clone()


class MemoryBaseCollector:
    """
    内存采集基类，为子类赋予内存采集的能力
    """

    @staticmethod
    def get_mem_usage() -> HardwareInfo:
        """
        获取当前系统内存使用率
        """
        return {
            "key": MEM_PCT_KEY,
            "name": "System Memory Utilization (%)",
            "value": psutil.virtual_memory().percent,
            "config": MEM_PCT_CONFIG,
        }

    @staticmethod
    def get_cur_proc_mem(proc: psutil.Process) -> HardwareInfoList:
        """
        获取当前进程的内存使用情况
        """
        mem_info = proc.memory_info()
        virtual_memory = psutil.virtual_memory()
        mem_proc: HardwareInfo = {
            "key": MEM_PROC_KEY,
            "name": "Process Memory In Use (non-swap) (MB)",
            "value": mem_info.rss / 1024 / 1024,
            "config": MEM_PROC_CONFIG,
        }
        mem_proc_pct: HardwareInfo = {
            "key": MEM_PROC_PCT_KEY,
            "name": "Process Memory Utilization (%)",
            "value": proc.memory_percent(),
            "config": MEM_PROC_PCT_CONFIG,
        }
        mem_proc_avail: HardwareInfo = {
            "key": PROC_MEM_AVAIL_KEY,
            "name": "Process Memory Available (non-swap) (MB)",
            "value": virtual_memory.available / 1024 / 1024,
            "config": PROC_MEM_AVAIL_CONFIG,
        }

        return [mem_proc, mem_proc_pct, mem_proc_avail]



#!/usr/bin/env python# -*- coding: utf-8 -*-
r"""
@DATE: 2023-11-30 21:20:13
@File: swanlab\env.py
@IDE: vscode
@Description:
    swanlab全局共用环境变量(运行时环境变量)
    除了utils和error模块，其他模块都可以使用这个模块
"""
import datetime
import enum
import io
import netrc
import os
import re
import sys
from enum import Enum
from pathlib import Path
from platform import platform
from typing import List, Union
from urllib.parse import urlparse


class SwanLabMode(Enum):
    """
    swanlab的解析模式，枚举类
    """

    DISABLED = "disabled"
    CLOUD = "cloud"
    OFFLINE = "offline"
    LOCAL = "local"

    @classmethod
    def list(cls) -> List[str]:
        """
        获取所有的枚举值
        :return: 所有的枚举值
        """
        return [item.value for item in cls]


class SwanLabEnv(enum.Enum):
    """
    swanlab环境变量枚举类，包含swankit的共享环境变量
    """

    SWANLAB_FOLDER = "SWANLAB_SAVE_DIR"
    """
    swanlab全局文件夹保存的路径，默认为用户主目录下的.swanlab文件夹
    """
    SWANLOG_FOLDER = "SWANLAB_LOG_DIR"
    """
    swanlab解析日志文件保存的路径，默认为当前运行目录的swanlog文件夹
    """
    MODE = "SWANLAB_MODE"
    """
    swanlab的解析模式，涉及操作员注册的回调，目前有四种：local、cloud、disabled、offline，默认为cloud
    大小写敏感
    """
    SWANBOARD_PROT = "SWANLAB_BOARD_PORT"
    """
    cli swanboard 服务端口
    """
    SWANBOARD_HOST = "SWANLAB_BOARD_HOST"
    """
    cli swanboard 服务地址
    """
    WEB_HOST = "SWANLAB_WEB_HOST"
    """
    swanlab云端环境的web地址
    """
    API_HOST = "SWANLAB_API_HOST"
    """
    swanlab云端环境的api地址
    """
    API_KEY = "SWANLAB_API_KEY"
    """
    云端api key，登录时会首先查找此环境变量，如果不存在，判断用户是否已登录，未登录则进入登录流程

    * 如果login接口传入字符串，此环境变量无效，此时相当于绕过 get_key 接口
    * 如果用户已登录，此环境变量的优先级高于本地存储登录信息
    """
    WORKSPACE = "SWANLAB_WORKSPACE"
    """
    swanlab的工作空间，默认为当前登录用户
    """
    PROJ_NAME = "SWANLAB_PROJ_NAME"
    """
    swanlab的项目名称
    """
    EXP_NAME = "SWANLAB_EXP_NAME"
    """
    swanlab的实验名称
    """
    RUN_ID = "SWANLAB_RUN_ID"
    """
    swanlab 实验运行 id，resume 时使用
    """
    RESUME = "SWANLAB_RESUME"
    """
    swanlab 实验是否为恢复运行，resume 时使用，设置为 true 则表示恢复运行
    """
    RUNTIME = "SWANLAB_RUNTIME"
    """
    swanlab的运行时环境，"user" "develop" "test" "test-no-cloud"，目前仅用于开发测试
    """
    WEBHOOK = "SWANLAB_WEBHOOK"
    """
    webhook地址。swanlab初始化完毕时，如果此环境变量存在，会调用此地址，发送消息。
    """
    DESCRIPTION = "SWANLAB_DESCRIPTION"
    """
    实验描述，用于为实验提供更详细的介绍或标注
    """
    JOB = "SWANLAB_JOB_TYPE"
    """
    实验任务类型，用于标注当前实验的任务类型，例如分类、回归等
    """
    GROUP = "SWANLAB_GROUP"
    """
    实验组，用于将实验划分到不同的组别，便于管理和区分
    """
    TAGS = "SWANLAB_TAGS"
    """
    实验标签，用于标注当前实验，多个标签用逗号分隔
    """
    DISABLE_GIT = "SWANLAB_DISABLE_GIT"
    """
    禁用Git功能，设置为true时不会采集Git信息
    """

    @staticmethod
    def is_hostname(value: str) -> bool:
        """
        判断是否为合法的主机名（支持 http/https 协议和端口号）
        :param value: 待判断的字符串
        :return: 是否为合法的主机名
        """
        # 解析 URL 以提取主机部分
        parsed_url = urlparse(value)
        if parsed_url.scheme in ["http", "https"]:
            value = parsed_url.hostname  # 只取域名部分

        # 处理 IP 地址
        if re.fullmatch(r'((25[0-5]|2[0-4][0-9]|1?[0-9]{1,2})(\.|$)){4}', value):
            return True

        # 处理域名（去掉端口）
        if re.fullmatch(r'^(?!-)([a-zA-Z0-9-]{1,63}\.)+[a-zA-Z]{2,63}$', value):
            return True

        return False

    @classmethod
    def set_default(cls):
        """
        设置默认的环境变量值，如果netrc文件存在，使用netrc中第一个machine的信息
        """
        # 从netrc文件中读取host信息
        path = os.path.join(get_save_dir(), ".netrc")
        web_host = "https://swanlab.cn"
        api_host = "https://api.swanlab.cn/api"
        if os.path.exists(path):
            nrc = netrc.netrc(path)
            for host, info in nrc.hosts.items():
                base_host = remove_host_suffix(host, "/api")
                web_host = info[0] if cls.is_hostname(info[0]) else base_host
                api_host = base_host + "/api"
                break

        envs = {
            cls.WEB_HOST.value: web_host,
            cls.API_HOST.value: api_host,
            cls.RUNTIME.value: "user",
        }
        for k, v in envs.items():
            os.environ.setdefault(k, v)

    @classmethod
    def check(cls):
        """
        检查环境变量的值是否为预期值中的一个
        :raises ValueError: 如果环境变量的值不在预期值中
        """
        envs = {
            cls.MODE.value: ["local", "cloud", "disabled", "offline"],
            cls.RUNTIME.value: ["user", "develop", "test", "test-no-cloud"],
        }
        for k, vs in envs.items():
            if k in os.environ and os.environ[k] not in vs:
                raise ValueError(f"Unknown value for {k}: {os.environ[k]}")

    @classmethod
    def list(cls) -> List[str]:
        """
        获取所有的枚举值
        :return: 所有的枚举值
        """
        return [item.value for item in cls]


def utc_time() -> datetime.datetime:
    """获取当前时间(UTC时区)"""
    return datetime.datetime.now(datetime.timezone.utc)


def create_time() -> str:
    """获取当前时间的iso字符串(UTC时区)"""
    return utc_time().isoformat()


def is_windows() -> bool:
    """判断当前操作系统是否是windows还是类unix系统，主要是路径分隔上的差别
    此外的系统会报错为 UnKnownSystemError
    :raise OSError: 未知系统错误，此时swanlab运行在未知系统上，这个系统不是windows或者类unix系统
    :return: True表示是windows系统，False表示是类unix系统
    """
    if sys.platform.startswith("win"):
        return True
    elif sys.platform.startswith("linux") or sys.platform.startswith("darwin"):
        return False
    raise OSError("Unknown system, not windows or unix-like system")


def is_macos() -> bool:
    """判断当前操作系统是否是macos
    :return: True表示是macos系统，False表示不是macos系统
    """
    return "mac" in platform().lower()


def get_mode() -> str:
    """
    获取当前的swanlab解析模式，如果没有设置，默认为cloud
    :raise ValueError: 未知的swanlab模式
    :return: swanlab的解析模式
    """
    mode = os.getenv(SwanLabEnv.MODE.value)
    if mode is None:
        mode = SwanLabMode.CLOUD.value
    mode = mode.lower()
    if mode not in SwanLabMode.list():
        raise ValueError(f"Unknown swanlab mode: {mode}")
    return mode


def get_save_dir() -> str:
    """
    获取存放swanlab全局文件的文件夹路径，如果不存在就创建
    此函数对应为SWANLAB_SAVE_FOLDER全局变量，如果没有设置，默认为用户主目录下的.swanlab文件夹
    执行此函数时，如果文件夹不存在，自动创建，但是出于安全考虑，不会自动创建父文件夹
    :raises
        :raise FileNotFoundError: folder的父目录不存在
        :raise NotADirectoryError: folder不是一个文件夹
    :return: swanlab全局文件夹保存的路径，返回处理后的绝对路径
    """
    folder = os.getenv(SwanLabEnv.SWANLAB_FOLDER.value)
    if folder is None:
        folder = os.path.join(os.path.expanduser("~"), ".swanlab")
    folder = os.path.abspath(folder)
    if not os.path.exists(os.path.dirname(folder)):
        raise FileNotFoundError(f"{os.path.dirname(folder)} not found")
    if not os.path.exists(folder):
        # 只创建当前文件夹，不创建父文件夹，之所以还要捕捉 FileExistsError，是因为在多线程或多进程环境下，可能会有多个线程或进程同时创建同一个文件夹
        # 比如：https://github.com/SwanHubX/SwanLab/issues/1033
        try:
            os.mkdir(folder)
        except FileExistsError:
            pass
    if not os.path.isdir(folder):
        raise NotADirectoryError(f"{folder} is not a directory")
    return folder


def get_swanlog_dir() -> str:
    """
    获取存放swanlog日志文件的文件夹路径
    此函数对应为SWANLAB_LOG_FOLDER全局变量，如果没有设置，默认为当前运行目录下的swanlog文件夹
    需要注意，此函数并不会保证文件夹的存在，但是会检查父文件夹是否存在以及folder是否是一个文件夹
    :raises
        :raise FileNotFoundError: folder的父目录不存在
        :raise NotADirectoryError: folder不是一个文件夹
    :return: swanlog日志文件保存的路径，返回处理后的绝对路径
    """
    folder = os.getenv(SwanLabEnv.SWANLOG_FOLDER.value)
    if folder is None:
        folder = os.path.join(os.getcwd(), "swanlog")
    folder = os.path.abspath(folder)
    if not os.path.exists(os.path.dirname(folder)):
        raise FileNotFoundError(f"{os.path.dirname(folder)} not found")
    if not os.path.exists(folder):
        return folder
    if not os.path.isdir(folder):
        raise NotADirectoryError(f"{folder} is not a directory")
    return folder


def create_swanlog_dir(logdir: Union[Path, str] = None):
    """
    获取swanlog文件夹，如果文件夹不存在则创建
    :param logdir: swanlog文件夹路径，默认为当前工作目录下的swanlog文件夹
    """
    if logdir is None:
        logdir = get_swanlog_dir()
    try:
        os.makedirs(logdir, exist_ok=True)
        if not os.access(logdir, os.W_OK):
            raise IOError(f"no write permission for path: {logdir}")
    except Exception as error:
        raise IOError(f"Failed to create or access logdir: {logdir}, error: {error}")
    # 如果logdir是空的，创建.gitignore文件，写入*
    if not os.listdir(logdir):
        with open(os.path.join(logdir, ".gitignore"), "w", encoding="utf-8") as f:
            f.write("*")
    return logdir


def in_jupyter() -> bool:
    """
    用于检测是否在 notebook jupyter 中运行
    :return: bool 是否在 notebook 中运行
    """
    try:
        # notebook 中会有 __IPYTHON__，而正常环境没有定义，所以 try
        # 'type: ignore': 可以让 pylance 忽略对变量定义的检查
        _ = __IPYTHON__  # type: ignore
        return True
    except NameError:
        return False


def is_interactive():
    """
    是否为可交互式环境（输入连接tty设备）
    特殊的环境：jupyter notebook
    """
    try:
        fd = sys.stdin.fileno()
        return os.isatty(fd) or in_jupyter()
    # 当使用capsys、capfd或monkeypatch等fixture来捕获或修改标准输入输出时，会抛出io.UnsupportedOperation
    # 多为测试情况，可交互
    except io.UnsupportedOperation:
        return True


def remove_host_suffix(host: str, suffix: str) -> str:
    """
    移除host的后缀
    :param host: 待处理的host
    :param suffix: 要移除的后缀
    :return: 处理后的host
    """
    host = host.rstrip()
    if len(suffix) == 0:
        return host
    if host.endswith(suffix):
        return host[: -len(suffix)]
    return host



"""
@author: cunyue
@file: __init__.py
@time: 2025/6/5 14:03
@description: 同步本地数据到云端
"""

import click

from swanlab.error import KeyFileError
from swanlab.package import HostFormatter, get_key
from swanlab.sync import sync as sync_logs


@click.command()
@click.argument(
    "path",
    nargs=-1,
    type=click.Path(
        exists=True,
        file_okay=False,
        dir_okay=True,
        readable=True,
        resolve_path=True,
    ),
    required=True,
)
@click.option(
    "--api-key",
    "-k",
    default=None,
    type=str,
    help="The API key to use for authentication. If not specified, it will use the default API key from the environment."
    "If specified, it will log in using this API key but will not save the key.",
)
@click.option(
    "--host",
    "-h",
    default=None,
    type=str,
    help="The host to sync the logs to. If not specified, it will use the default host.",
)
@click.option(
    "--workspace",
    "-w",
    default=None,
    type=str,
    help="The workspace to sync the logs to. If not specified, it will use the default workspace.",
)
@click.option(
    "--project",
    "-p",
    default=None,
    type=str,
    help="The project to sync the logs to. If not specified, it will use the default project.",
)
@click.option(
    "--id",
    "-i",
    default=None,
    type=str,
    help="The experiment ID to sync the logs to. It can only be used when the path is a single directory."
    "For more details, see https://docs.swanlab.cn/api/cli-swanlab-sync.html",
)
def sync(path, api_key, workspace, project, host, id):
    """
    Synchronize local logs to the cloud.
    """
    # 1. 检查 host 参数是否符合规范，并注入环境变量中
    HostFormatter(host)()
    # 2. 尝试获取 API key
    try:
        if api_key is None:
            api_key = get_key()
    except KeyFileError:
        pass
    # 2. 同步数据
    for p in path:
        sync_logs(p, workspace=workspace, project=project, id=id, api_key=api_key, raise_error=len(path) == 1)



"""
@author: cunyue
@file: verify.py
@time: 2025/9/23 11:06
@description: 验证用户当前登录状态
"""

import click
from rich.text import Text

from swanlab.core_python import auth
from swanlab.error import KeyFileError
from swanlab.log import swanlog
from swanlab.package import get_key


@click.command()
def verify():
    """Verify the current login status."""
    try:
        key = get_key()
    except KeyFileError:
        raise click.ClickException("You are not verified. Please use `swanlab login` to login.")
    login_info = auth.code_login(key, save_key=False)
    swanlog.info('You are logged into', login_info.web_host, 'as', Text(login_info.username, "bold"))



"""
@DATE: 2024/5/5 20:22
@File: callback_cloud.py
@IDE: pycharm
@Description:
    云端回调
"""

import shutil
from concurrent.futures.thread import ThreadPoolExecutor
from typing import Optional

from rich.status import Status
from rich.text import Text

from swanlab.core_python import auth
from swanlab.data.callbacker.callback import SwanLabRunCallback
from swanlab.env import in_jupyter
from swanlab.log import swanlog
from swanlab.toolkit import (
    RuntimeInfo,
    MetricInfo,
    ColumnInfo,
)
from . import utils as U
from ..porter import Mounter
from ..run import get_run
from ...core_python import *
from ...log.type import LogData


class CloudPyCallback(SwanLabRunCallback):
    login_info: Optional[auth.LoginInfo] = None

    def __init__(self):
        super().__init__()
        self.executor = ThreadPoolExecutor(max_workers=1)

    def __str__(self):
        return "SwanLabCloudPyCallback"

    @staticmethod
    def _create_client():
        try:
            http = get_client()
        except ValueError:
            swanlog.debug("Login info is None, get login info.")
            login_info = CloudPyCallback.login_info
            if login_info is not None:
                # 如果有登录信息，则使用该信息创建客户端
                http = create_client(login_info)
                CloudPyCallback.login_info = None
            else:
                # 如果没有登录信息，则需要用户登录
                # 但是不保存登录信息到本地
                http = create_client(auth.create_login_info(save=False))
        return http

    @staticmethod
    def _converter_summarise_metric():
        pass

    def on_init(self, *args, **kwargs):
        _ = self._create_client()
        # 检测是否有最新的版本
        U.check_latest_version()
        with Status("Creating experiment...", spinner="dots"):
            with Mounter() as mounter:
                mounter.execute()

    def _terminal_handler(self, log_data: LogData):
        self.porter.trace_log(log_data)

    def on_run(self, *args, **kwargs):
        self.porter.open_for_trace(sync=False)
        # 注册终端代理和系统回调
        self._start_terminal_proxy()
        self._register_sys_callback()
        # 打印实验开始信息，在 cloud 模式下如果没有开启 backup 的话不打印“数据保存在 xxx”的信息
        U.print_train_begin(run_dir=self.run_store.run_dir)
        http = get_client()
        swanlog.info("👋 Hi ", Text(http.username, "bold default"), ",welcome to swanlab!", sep="")
        swanlog.info("Syncing run", Text(self.run_store.run_name, "yellow"), "to the cloud")
        experiment_url = U.print_cloud_web()
        # 在Jupyter Notebook环境下，显示按钮
        if in_jupyter():
            U.show_button_html(experiment_url)

    def on_runtime_info_update(self, r: RuntimeInfo, *args, **kwargs):
        self.porter.trace_runtime_info(r)

    def on_column_create(self, column_info: ColumnInfo, *args, **kwargs):
        self.porter.trace_column(column_info)

    def on_metric_create(self, metric_info: MetricInfo, *args, **kwargs):
        # 有错误就不上传
        if metric_info.error:
            return
        self.porter.trace_metric(metric_info)

    def on_stop(self, error: str = None, *args, **kwargs):
        success = get_run().success
        # FIXME 等合并 swankit 以后优化一下 interrupt 的传递问题
        interrupt = kwargs.get("interrupt", False)
        http = get_client()
        if http.pending:
            swanlog.warning("This run was destroyed but it is pending!")
        # 打印信息
        U.print_cloud_web()
        error_epoch = swanlog.epoch + 1
        self._unregister_sys_callback()
        self.porter.close_trace(success, error=error, epoch=error_epoch)
        # 更新实验状态，在此之后实验会话关闭
        http.update_state(success, interrupt=interrupt)
        reset_client()
        if not self.user_settings.backup:
            shutil.rmtree(self.run_store.run_dir, ignore_errors=True)



try:
    import catboost
except ImportError:
    raise RuntimeError(
        "This module requires `catboost` to be installed. "
        "Please install it with command: pip install catboost"
    )

from typing import Any, Dict, List, NamedTuple, Optional
import swanlab


class IterationInfo(NamedTuple):
    iteration: int
    metrics: Dict[str, Dict[str, List[float]]]


class SwanLabCallback:
    def __init__(self, params: Optional[Dict[str, Any]] = None):
        """
        Initializes the SwanLabCallback for CatBoost.

        :params: Optional dictionary of parameters to log.
        """
        if swanlab.get_run() is None:
            raise RuntimeError(
                "You must call swanlab.init() before using SwanLabCallback."
            )
        swanlab.config["FRAMEWORK"] = "catboost"  # type: ignore
        if params and isinstance(params, dict):
            swanlab.config.update(params)

    def update_config(self, config: Dict[str, Any]):
        swanlab.config.update(config)

    def after_iteration(self, info: IterationInfo) -> bool:
        """
        Called after each iteration. Logs metrics and handles first-time setup.
        :return: True/False to continue training.
        """
        # Log metrics with data_name_metric_name format based on CatBoost JSON structure
        if info.metrics:
            for stage_name, metrics in info.metrics.items():
                for metric_name, values in metrics.items():
                    if values:
                        swanlab.log({f"{stage_name}_{metric_name}": values[-1]})

        return True



"""
@author: cunyue
@file: __init__.py.py
@time: 2025/6/22 19:46
@description: 数据搬运工，在 callbacker 中被实例化，支持：
1. 数据上报（可选）：将数据上报到 SwanLab 服务器
2. 数据备份（必须）：将数据备份到本地文件系统
3. 读取本地备份并上报：读取本地备份文件并上报到 SwanLab 服务器

其中数据上报可选择使用 go 后端或者 python 线程
SwanLab 不生产数据，我们只是 AI 训练的搬运工，祝愿所有训练者跑出预期的 loss :)
"""

import os
from concurrent.futures import ThreadPoolExecutor
from typing import Optional, Literal, List, Union, Tuple

import wrapt

from swanlab.core_python import get_client
from swanlab.core_python.uploader import ColumnModel, ScalarModel, MediaModel, LogModel
from swanlab.core_python.uploader.thread import ThreadPool, UploadType
from swanlab.data.store import RunStore, get_run_store, reset_run_store
from swanlab.env import create_time
from swanlab.error import ValidationError
from swanlab.log.type import LogData
from swanlab.proto.v0 import Log, Header, Project, Experiment, Column, Metric, BaseModel, Runtime, Footer, Media, Scalar
from swanlab.toolkit import MetricInfo, ColumnInfo, RuntimeInfo
from .datastore import DataStore
from .mounter import Mounter
from .utils import filter_metric, filter_epoch, filter_column
from ...toolkit.models.log import LogContent

__all__ = ['DataPorter', 'Mounter']


def traced():
    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        if instance is not None and getattr(instance, "_mode") != 1:
            raise RuntimeError("ProtoTransfer has not been started for tracing. Call oopen_for_trace() first.")
        return wrapped(*args, **kwargs)

    return wrapper


def synced():
    """
    标记需要同步上传的函数
    """

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        if instance is not None and getattr(instance, "_mode") != 2:
            raise RuntimeError("ProtoTransfer has not been started for sync upload. Call open_for_sync() first.")
        return wrapped(*args, **kwargs)

    return wrapper


def backup():
    """
    标记需要备份的函数，并将函数的返回结果存储到备份文件中
    """

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        result: Union[List[BaseModel], BaseModel] = wrapped(*args, **kwargs)
        if result is not None:
            try:
                f = getattr(instance, "_f")
                if isinstance(result, list):
                    [f.write(item.to_record()) for item in result]
                else:
                    f.write(result.to_record())
            except ValueError:
                # 写入备份文件失败，可能是因为没有开启备份模式或者备份文件未打开
                # TODO: 记录本地日志
                pass
            except TypeError:
                # 目前此错误会发生在与 transformers 集成时 ctrl + c，tqdm 没有立即退出
                # TODO: 记录本地日志
                pass
        return result

    return wrapper


def async_io():
    """
    标记需要在其他线程处理的函数
    """

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        executor: ThreadPoolExecutor = getattr(instance, "_executor", None)
        # 如果没有设置 executor，则直接执行函数
        if executor is None:
            return wrapped(*args, **kwargs)
        # 与 https://github.com/SwanHubX/SwanLab/issues/889 相同的问题
        # 在回调中线程池已经关闭，我们需要在主线程中执行
        if executor._shutdown:
            return wrapped(*args, **kwargs)
        # 否则将函数提交到线程池中执行
        executor.submit(wrapped, *args, **kwargs)

    return wrapper


class DataPorter:
    _instance: Optional['DataPorter'] = None
    _run_store: Optional[RunStore] = None

    def __init__(self):
        # 上传线程池，可以选择不开启
        self._pool: Optional[ThreadPool] = None
        # 数据存储句柄
        self._f = DataStore()
        # 当前模式
        # 0: 不开启任何模式
        # 1: 实验日志跟踪模式
        # 2: 同步上传模式，用于本地日志上报
        self._mode: Literal[0, 1, 2] = 0
        # 工作线程池，开启后一些方法会运行在子线程中
        self._executor: Optional[ThreadPoolExecutor] = None
        self._closed = False

        # ---------------------------------- 同步时用到的参数 ----------------------------------
        self._header: Optional[Header] = None
        self._project: Optional[Project] = None
        self._experiment: Optional[Experiment] = None
        self._logs: List[Log] = []
        self._runtime: Runtime = Runtime(
            conda_filename=None,
            requirements_filename=None,
            metadata_filename=None,
            config_filename=None,
        )
        self._columns: List[Column] = []
        self._scalars: List[Scalar] = []
        self._medias: List[Media] = []
        self._footer: Optional[Footer] = None

    def __new__(cls):
        """
        开启单例模式，确保同时只有一个 ProtoTransfer 实例存在，即代表一个实验会话
        创建会话之前，client 必须存在
        """
        if cls._instance is not None:
            raise RuntimeError("DataPorter instance already exists, cannot create a new one.")
        run_store = get_run_store()
        cls._run_store = run_store
        cls._instance = super(DataPorter, cls).__new__(cls)
        return cls._instance

    def _set_mode(self, mode: Literal[0, 1, 2]):
        """
        设置当前模式，只允许从0设置为其他模式
        """
        assert mode in [0, 1, 2], "Mode must be one of [0, 1, 2]."
        assert self._mode == 0, "DataPorter is already in use, cannot change mode."
        assert self._closed is False, "DataPorter has already rested, cannot change mode."
        self._mode = mode

    def open_for_trace(self, sync: bool = False, backend: Literal['go', 'python', 'none'] = 'python'):
        """
        开启日志传输器以实验日志跟踪，创建数据存储句柄
        此函数在同一实例中只能调用一次，后续调用会抛出异常
        :param sync: 是否同步上传数据，默认为 False，表示异步上传
        :param backend: 上传后端类型，默认为 'python'，可选 'go' 或 'none', none 表示不开启上传
        :raises RuntimeError: 如果已经开启了跟踪模式或者别的模式，则抛出异常
        """
        assert self._run_store.media_dir, "Media directory must be set before creating ProtoTransfer instance"
        assert self._run_store.file_dir, "File directory must be set before creating ProtoTransfer instance"
        assert self._run_store.backup_file, "Backup file must be set before creating ProtoTransfer instance"

        assert self._mode == 0, "DataPorter is already in use, cannot open for trace again."
        assert self._closed is False, "DataPorter has already rested, cannot open for trace again."

        self._f.open_for_write(self._run_store.backup_file)
        if backend == 'python':
            # 检查是否已经创建 client
            try:
                get_client()
            except ValueError:
                raise ValueError("Client not initialized when creating ProtoTransfer instance")
            self._pool = ThreadPool()
        elif backend == 'go':
            raise NotImplementedError("swanlab-core is not ready yet.")
        elif backend == 'none':
            pass
        else:
            raise ValueError(f"Unsupported backend: {backend}")

        if not sync:
            self._executor = ThreadPoolExecutor(max_workers=1)

        # 写入备份文件头
        self._f.write(
            Header.model_validate(
                {
                    "create_time": create_time(),
                    "backup_type": "DEFAULT",
                }
            ).to_record()
        )
        # 项目
        project = self._run_store.project
        workspace = self._run_store.workspace
        visibility = self._run_store.visibility
        # 实验
        id = self._run_store.run_id
        name = self._run_store.run_name
        colors = self._run_store.run_colors
        description = self._run_store.description
        tags = self._run_store.tags
        self._f.write(
            Project.model_validate(
                {
                    "name": project,
                    "workspace": workspace,
                    "public": visibility,
                }
            ).to_record()
        )
        self._f.write(
            Experiment.model_validate(
                {
                    "id": id,
                    "name": name,
                    "colors": colors,
                    "description": description,
                    "tags": tags,
                }
            ).to_record()
        )
        self._set_mode(1)

    def _publish(self, *args, **kwargs):
        """
        发布数据到上传线程池，如果未开启线程池，则不执行任何操作
        """
        if self._pool is not None:
            self._pool.queue.put(*args, **kwargs)

    @async_io()
    @backup()
    @traced()
    def trace_column(self, data: ColumnInfo) -> BaseModel:
        """
        追踪列数据
        """
        column = Column.from_column_info(data)
        self._publish((UploadType.COLUMN, [column.to_column_model()]))
        return column

    @async_io()
    @backup()
    @traced()
    def trace_metric(self, data: MetricInfo) -> BaseModel:
        """
        追踪指标数据
        """
        assert data.error is None, "MetricInfo must not have error, if it has error, do not upload it."
        if data.column_info.chart_type == data.column_info.chart_type.LINE:
            # 标量
            scalar = Metric.from_metric_info(data)
            self._publish((UploadType.SCALAR_METRIC, [scalar.to_scalar_model()]))
            return scalar
        else:
            # 媒体
            media = Metric.from_metric_info(data)
            # 写入媒体信息
            if data.metric_buffers is not None:
                for i, r in enumerate(data.metric_buffers):
                    if r is None:
                        continue
                    # 组合路径
                    path = os.path.join(data.swanlab_media_dir, data.column_info.kid)
                    os.makedirs(path, exist_ok=True)
                    if data.metric is None:
                        # TODO: 记录日志
                        raise ValueError("MetricInfo must have metric data when uploading media.")
                    # 写入数据
                    with open(os.path.join(path, data.metric["data"][i]), "wb") as f:
                        f.write(r.getvalue())
            self._publish((UploadType.MEDIA_METRIC, [media.to_media_model(data.swanlab_media_dir)]))
            return media

    @async_io()
    @backup()
    @traced()
    def trace_runtime_info(self, data: RuntimeInfo) -> BaseModel:
        """
        追踪运行时信息
        """
        runtime = Runtime.from_runtime_info(data)
        # 写入运行时信息到数据存储
        file_dir = self._run_store.file_dir
        if data.requirements is not None:
            data.requirements.write(file_dir)
        if data.metadata is not None:
            data.metadata.write(file_dir)
        if data.config is not None:
            data.config.write(file_dir)
        if data.conda is not None:
            data.conda.write(file_dir)
        self._publish((UploadType.FILE, [runtime.to_file_model(file_dir)]))
        return runtime

    @async_io()
    @backup()
    @traced()
    def trace_log(self, data: LogData) -> List[BaseModel]:
        """
        追踪日志数据，日志数据比较特殊，一次可能好几行
        """
        logs = Log.from_log_data(data)
        self._publish((UploadType.LOG, [log.to_log_model() for log in logs]))
        return logs

    @traced()
    def close_trace(self, success: bool, error: str = None, epoch: int = None):
        """
        停止日志跟踪，清理相关资源
        """
        assert self._closed is False, "DataPorter has already rested, cannot close trace again."
        assert self._mode == 1, "DataPorter is not in trace mode, cannot close trace."

        # 上传错误日志
        if error is not None:
            log = Log.model_validate({"level": "ERROR", "message": error, "create_time": create_time(), "epoch": epoch})
            self._publish((UploadType.LOG, [log.to_log_model()]))
            # 备份日志
            self._f.write(log.to_record())
        # 停止worker
        if self._executor is not None:
            self._executor.shutdown(wait=True)
        # 停止上传线程池
        if self._pool is not None:
            self._pool.finish()
        # 写入结束标志
        footer = Footer.model_validate({"create_time": create_time(), "success": success})
        self._f.write(footer.to_record())
        self._f.ensure_flushed()
        self._f.close()
        self._closed = True
        DataPorter._reset()

    @classmethod
    def _reset(cls):
        """重置单例实例，允许创建新的DataPorter"""
        cls._instance = None
        cls._run_store = None

    def open_for_sync(self, run_dir: str, backend: Literal['python', 'go'] = 'python') -> "DataPorter":
        """
        开启同步模式，此函数应该与 __enter__() 一起使用
        """
        self._run_store.run_dir = run_dir
        assert self._run_store.media_dir, "Media directory must be set before creating ProtoTransfer instance"
        assert self._run_store.file_dir, "File directory must be set before creating ProtoTransfer instance"
        assert self._run_store.backup_file, "Backup file must be set before creating ProtoTransfer instance"
        backup_file = self._run_store.backup_file
        assert os.path.isfile(backup_file), f"Backup file {backup_file} does not exist."
        self._f.open_for_scan(backup_file)
        if backend == 'python':
            self._pool = ThreadPool()
        elif backend == 'go':
            raise NotImplementedError("swanlab-core is not ready yet.")
        else:
            raise ValueError(f"Unsupported backend for sync: {backend}")
        self._set_mode(2)
        return self

    @synced()
    def __enter__(self):
        """
        开启本地数据备份和上传线程池
        使用方式为：
        >>> backup_file = "path/to/backup/file"
        >>> with DataPorter().open_for_sync(backup_file) as porter:
        >>>    porter.parse()
        >>>    ... # 创建实验
        >>>    porter.synchronize()
        """
        assert self._mode == 2, "DataPorter is already in use, cannot open for sync."
        assert self._closed is False, "DataPorter has already rested, cannot open for sync."
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        reset_run_store()
        self._closed = True
        DataPorter._reset()
        # 出现错误则抛出异常
        if exc_type is not None:
            raise exc_val

    @synced()
    def parse(self) -> Tuple[Project, Experiment]:
        """
        解析备份文件中的记录，必须在 open_for_sync() 后调用
        """
        try:
            for record in self._f:
                if record is not None:
                    self._parse_record(record)
        except ValidationError:
            # 略过坏的记录，直接退出
            # TODO 写入日志文件
            pass
        # 检查是否所有必要的记录都已解析
        assert self._header is not None, "Header not parsed"
        # 检查备份文件
        assert (
            self._header.backup_type == "DEFAULT"
        ), f"Backup type mismatch: {self._header.backup_type}, please update your swanlab package."
        assert self._project is not None, "Project not parsed"
        assert self._experiment is not None, "Experiment not parsed"
        return self._project, self._experiment

    def _parse_record(self, data: str):
        assert self._mode == 2, "Must parse records in sync mode (mode=2)."
        assert self._closed is False, "DataPorter has already rested, cannot parse."
        record = BaseModel.from_record(data)
        if isinstance(record, Header):
            assert self._header is None, "Header already parsed"
            self._header = record
            return
        if isinstance(record, Project):
            assert self._project is None, "Project already parsed"
            self._project = record
            return
        if isinstance(record, Experiment):
            assert self._experiment is None, "Experiment already parsed"
            self._experiment = record
            return
        if isinstance(record, Log):
            self._logs.append(record)
            return
        if isinstance(record, Runtime):
            if record.conda_filename is not None:
                self._runtime.conda_filename = record.conda_filename
            if record.requirements_filename is not None:
                self._runtime.requirements_filename = record.requirements_filename
            if record.metadata_filename is not None:
                self._runtime.metadata_filename = record.metadata_filename
            if record.config_filename is not None:
                self._runtime.config_filename = record.config_filename
            return
        if isinstance(record, Column):
            self._columns.append(record)
            return
        if isinstance(record, Scalar):
            self._scalars.append(record)
            return
        if isinstance(record, Media):
            self._medias.append(record)
            return
        if isinstance(record, Footer):
            assert self._footer is None, "Footer already parsed"
            self._footer = record
            return
        raise ValueError(f"Unknown record type: {type(record)}")

    def close(self):
        """
        关闭实例，此函数用于没有开启 trace 和 sync 时关闭实例
        """
        assert self._closed is False, "DataPorter has already rested, cannot close."
        assert self._mode == 0, "DataPorter is in use, cannot simply close."
        DataPorter._reset()

    @synced()
    def synchronize(self):
        """
        同步上传数据到 SwanLab 服务器，必须在 open_for_sync() 后调用
        NOTE: 执行此函数后，实验会话被关闭
        """
        assert self._mode == 2, "Must synchronize in sync mode (mode=2)."
        assert self._closed is False, "DataPorter has already rested, cannot synchronize."
        # 同步上传数据到 SwanLab 服务器
        # 1. 上传文件（配置、运行时）
        self._publish((UploadType.FILE, [self._runtime.to_file_model(file_dir=self._run_store.file_dir)]))
        # 2. 上传列信息
        columns = filter(self._filter_column_by_key, [column.to_column_model() for column in self._columns])
        self._publish((UploadType.COLUMN, list(columns)))
        # 3. 上传标量指标
        scalars = filter(self._filter_scalar_by_step, [scalar.to_scalar_model() for scalar in self._scalars])
        self._publish((UploadType.SCALAR_METRIC, list(scalars)))
        # 4. 上传媒体指标
        medias = filter(
            self._filter_media_by_step, [media.to_media_model(self._run_store.media_dir) for media in self._medias]
        )
        self._publish((UploadType.MEDIA_METRIC, list(medias)))
        # 5. 上传日志
        logs = [log.to_log_model() for log in self._logs]
        for ls in logs:
            ls['contents'] = list(filter(self._filter_log_by_epoch, ls['contents']))
        self._publish((UploadType.LOG, logs))
        # 6. 等待上传完毕
        self._pool.finish()
        # 7. 同步实验状态
        client = get_client()
        if self._footer is None:
            client.update_state(success=False)
        else:
            client.update_state(self._footer.success, self._footer.create_time)

    def _filter_column_by_key(self, column: ColumnModel) -> bool:
        """
        筛选列指标，排除已经上传的列，此函数用于 filter 高阶函数
        :param column: 列指标数据
        :return 是否需要被保留
        """
        return filter_column(column.key, self._run_store.metrics)

    def _filter_scalar_by_step(self, metric: ScalarModel) -> bool:
        """
        筛选标量指标数据，排除已经上传的指标
        :param metric: 指标数据
        :return: 是否需要被保留
        """
        return filter_metric(metric.key, metric.step, self._run_store.metrics)

    def _filter_media_by_step(self, metric: MediaModel) -> bool:
        """
        筛选媒体数据，排除已经上传的媒体
        :param metric : 媒体数据
        :return: 是否需要被保留
        """
        return filter_metric(metric.key, metric.step, self._run_store.metrics)

    def _filter_log_by_epoch(self, log: LogContent) -> LogModel:
        """
        筛选日志数据，排除已经上传的日志
        :param log: 日志数据
        :return: 是否需要被保留
        """
        return filter_epoch(log['epoch'], self._run_store.log_epoch)



"""
Docs: https://docs.swanlab.cn/guide_cloud/integration/integration-huggingface-accelerate.html

For adaptation to the huggingface accelerate. You can used SwanLab as your tracker, experiment logs can be uploaded to
SwanLab or viewed using the local version of SwanLab. Detailed of used swanlab in accelerate train scripts are as follows:
------train.py in accelerate------
...
from swanlab.integration.accelerate import SwanLabTracker
...
tracker = SwanLabTracker("some_run_name")
accelerator = Accelerator(log_with=[tracker])
...
---------------------------------
These also can be mixed with existing trackers, including with "all":
------train.py in accelerate------
...
from swanlab.integration.accelerate import SwanLabTracker
...
tracker = SwanLabTracker("some_run_name")
accelerator = Accelerator(log_with=[tracker, "all"])
...
---------------------------------
"""

import os
from typing import Any, Dict, List, Optional, Union

import swanlab

try:
    from accelerate.tracking import GeneralTracker, on_main_process
    from accelerate.logging import get_logger
except ImportError:
    raise RuntimeError(
        "This contrib module requires Accelerate to be installed. "
        "Please install it with command: \n pip install accelerate"
    )


logger = get_logger(__name__)


class SwanLabTracker(GeneralTracker):
    """
    A `Tracker` class that supports `swanlab`. Should be initialized at the start of your script.

    Args:
        run_name (`str`):
            The name of the experiment run.
        **kwargs (additional keyword arguments, *optional*):
            Additional key word arguments passed along to the `swanlab.init` method.
    """

    name = "swanlab"
    requires_logging_directory = False
    main_process_only = True  # it must be True in tracker module

    def __init__(self, run_name: str, **kwargs):
        super().__init__()
        self.run_name = run_name
        self.init_kwargs = kwargs
        self.start()  # auto start swanlab when instantiation tracker

    @on_main_process
    def start(self):
        if hasattr(self, "run") and self.run is not None:
            return

        self.run = swanlab.init(project=self.run_name, **self.init_kwargs)
        swanlab.config["FRAMEWORK"] = "accelerate"  # add accelerate logo in config
        logger.debug(f"Initialized SwanLab project {self.run_name}")
        logger.debug(
            "Make sure to log any initial configurations with `self.store_init_configuration` before training!"
        )

    @property
    def tracker(self):
        return self.run

    @on_main_process
    def store_init_configuration(self, values: dict):
        """
        Logs `values` as hyperparameters for the run. Should be run at the beginning of your experiment.

        Args:
            values (Dictionary `str` to `bool`, `str`, `float` or `int`):
                Values to be stored as initial hyperparameters as key-value pairs. The values need to have type `bool`,
                `str`, `float`, `int`, or `None`.
        """
        import swanlab

        swanlab.config.update(values, allow_val_change=True)
        logger.debug("Stored initial configuration hyperparameters to SwanLab")

    @on_main_process
    def log(self, values: dict, step: Optional[int] = None, **kwargs):
        """
        Logs `values` to the current run.

        Args:
        data : Dict[str, DataType]
            Data must be a dict.
            The key must be a string with 0-9, a-z, A-Z, " ", "_", "-", "/".
            The value must be a `float`, `float convertible object`, `int` or `swanlab.data.BaseType`.
        step : int, optional
            The step number of the current data, if not provided, it will be automatically incremented.
        If step is duplicated, the data will be ignored.
            kwargs:
                Additional key word arguments passed along to the `swanlab.log` method. Likes:
                    print_to_console : bool, optional
                        Whether to print the data to the console, the default is False.
        """
        self.run.log(values, step=step, **kwargs)
        logger.debug("Successfully logged to SwanLab")

    @on_main_process
    def log_images(self, values: dict, step: Optional[int] = None, **kwargs):
        """
        Logs `images` to the current run.

        Args:
            values (Dictionary `str` to `List` of `np.ndarray` or `PIL.Image`):
                Values to be logged as key-value pairs. The values need to have type `List` of `np.ndarray` or
            step (`int`, *optional*):
                The run step. If included, the log will be affiliated with this step.
            kwargs:
                Additional key word arguments passed along to the `swanlab.log` method. Likes:
                    print_to_console : bool, optional
                        Whether to print the data to the console, the default is False.
        """
        import swanlab

        for k, v in values.items():
            self.log({k: [swanlab.Image(image) for image in v]}, step=step, **kwargs)
        logger.debug("Successfully logged images to SwanLab")

    @on_main_process
    def finish(self):
        """
        Closes `swanlab` writer
        """
        self.run.finish()
        logger.debug("SwanLab run closed")



"""
@author: cunyue
@file: test_utils.py
@time: 2025/9/9 12:32
@description: 测试utils.py中的函数
"""

import os

from swanlab.data.utils import _load_list_from_env


class TestLoadListFromEnv:
    def test_loads_tags_from_env_variable(self):
        os.environ["SWANLAB_TAGS"] = "tag1, tag2, tag3"
        result = _load_list_from_env("SWANLAB_TAGS", None)
        assert result == ["tag1", "tag2", "tag3"]

    def test_returns_value_if_provided(self):
        result = _load_list_from_env("SWANLAB_TAGS", ["custom_tag"])
        assert result == ["custom_tag"]

    def test_returns_none_if_env_variable_not_set(self):
        if "SWANLAB_TAGS" in os.environ:
            del os.environ["SWANLAB_TAGS"]
        result = _load_list_from_env("SWANLAB_TAGS", None)
        assert result is None

    def test_handles_empty_env_variable(self):
        os.environ["SWANLAB_TAGS"] = ""
        result = _load_list_from_env("SWANLAB_TAGS", None)
        assert result == []

    def test_strips_whitespace_from_tags(self):
        os.environ["SWANLAB_TAGS"] = " tag1 , tag2 , tag3 "
        result = _load_list_from_env("SWANLAB_TAGS", None)
        assert result == ["tag1", "tag2", "tag3"]



"""
@author: cunyue
@file: config.py
@time: 2025/10/10 22:01
@description: 配置模型，包括：
- RequirementInfo: 依赖信息
- MetadataInfo: 系统信息
- ConfigInfo: 用户配置信息
- CondaInfo: conda环境信息
- RuntimeInfo: 运行时信息，包含上述所有信息
"""

import json
import os
from abc import ABC, abstractmethod
from typing import Optional

import yaml

__all__ = ["RuntimeInfo"]


class InfoWriter(ABC):

    def __init__(self, info):
        self.info = info

    @abstractmethod
    def write(self, path: str):
        """
        写入到本地文件的方法
        :param path: 写入的路径，不包含文件名
        """
        pass

    @abstractmethod
    def dumps(self):
        """
        将信息转换为字符串
        """
        pass

    @abstractmethod
    def to_dict(self):
        """
        将信息转换为字典
        """
        pass


class RequirementInfo(InfoWriter):
    """
    依赖信息
    """

    def __init__(self, info: str):
        super().__init__(info)
        self.name = "requirements.txt"

    def write(self, path: str):
        with open(os.path.join(path, self.name), "w", encoding="utf-8") as f:
            f.write(self.info)

    def dumps(self):
        return self.info

    def to_dict(self):
        raise NotImplementedError("RequirementInfo has no to_dict method")


class MetadataInfo(InfoWriter):
    """
    系统信息
    """

    def __init__(self, info: dict):
        super().__init__(info)
        self.name = "swanlab-metadata.json"

    def write(self, path: str):
        with open(os.path.join(path, self.name), "w", encoding="utf-8") as f:
            f.write(self.dumps())

    def dumps(self):
        return json.dumps(self.to_dict(), ensure_ascii=False)

    def to_dict(self):
        return self.info


class ConfigInfo(InfoWriter):
    """
    配置信息
    """

    def __init__(self, info: dict):
        super().__init__(info)
        self.name = "config.yaml"
        self.__data = None

    def write(self, path: str):
        with open(os.path.join(path, self.name), "w", encoding="utf-8") as f:
            f.write(self.dumps())

    def dumps(self):
        return yaml.dump(self.to_dict(), allow_unicode=True)

    def to_dict(self):
        """
        返回配置信息的字典形式
        """
        # 遍历每一个配置项，值改为value，增加desc和sort字段
        # 原因是序列化时可能会丢失key的排序信息，所以这里增加一个sort字段
        # 没有在__init__中直接修改是因为可能会有其他地方需要原始数据，并且会丢失一些性能
        if self.__data is not None:
            return self.__data
        self.__data = {k: {"value": v, "sort": i, "desc": ""} for i, (k, v) in enumerate(self.info.items())}
        return self.__data


class CondaInfo(InfoWriter):
    """
    conda环境信息
    """

    def __init__(self, info: str):
        super().__init__(info)
        self.name = "conda.yaml"

    def write(self, path: str):
        with open(os.path.join(path, self.name), "w", encoding="utf-8") as f:
            f.write(self.dumps())

    def dumps(self):
        return self.info

    def to_dict(self):
        raise NotImplementedError("CondaInfo has no to_dict method")


class RuntimeInfo:
    """
    运行时信息，包括系统信息，依赖信息等
    如果某些信息为None，代表没有传入配置
    """

    def __init__(self, requirements: str = None, metadata: dict = None, config: dict = None, conda: str = None):
        """
        :param requirements: python依赖信息
        :param metadata: 系统信息
        :param config: 上传的配置信息
        """
        self.requirements: Optional[RequirementInfo] = (
            RequirementInfo(requirements) if requirements is not None else None
        )

        self.metadata: Optional[MetadataInfo] = MetadataInfo(metadata) if metadata is not None else None

        self.config: Optional[ConfigInfo] = ConfigInfo(config) if config is not None else None

        self.conda: Optional[CondaInfo] = CondaInfo(conda) if conda is not None else None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2025/5/1 17:00
@File: main.py
@IDE: pycharm
@Description:
    SwanLab OpenAPI模块
"""
from typing import Dict, List, Union

from swanlab.api.base import ApiHTTP, get_logger
from swanlab.api.experiment import ExperimentAPI
from swanlab.api.group import GroupAPI
from swanlab.api.project import ProjectAPI
from swanlab.api.types import ApiResponse, Experiment, Project
from swanlab.core_python import auth
from swanlab.error import KeyFileError
from swanlab.log.log import SwanLog
from swanlab.package import get_key

try:
    from pandas import DataFrame
except ImportError:
    DataFrame = None


class OpenApi:
    def __init__(self, api_key: str = "", log_level: str = "info"):
        self.__logger: SwanLog = get_logger(log_level)

        if api_key:
            self.__logger.debug("Using API key", api_key)
            self.__key = api_key
            self.login_info = auth.code_login(self.__key, False)
        else:
            self.__logger.debug("Using existing key")
            try:
                self.__key = get_key()
            except KeyFileError as e:
                self.__logger.error("To use SwanLab OpenAPI, please login first.")
                raise RuntimeError("Not logged in.") from e
            self.login_info = auth.code_login(self.__key, False)

        self.username = self.login_info.username
        self.http: ApiHTTP = ApiHTTP(self.login_info)
        self.service = self.http.service

        self.group = GroupAPI(self.http)
        self.experiment = ExperimentAPI(self.http)
        self.project = ProjectAPI(self.http)

    def list_workspaces(self) -> ApiResponse[List[Dict]]:
        """
        获取当前用户的所有工作空间(Group)

        Returns:
            ApiResponse[List]:
                - code (int): HTTP 状态码
                - errmsg (str): 错误信息, 仅在请求有错误时非空
                - data (List[Dict]): 一个列表, 其中每个元素是一个字典, 包含相应工作空间的基础信息:
                    - name (str): 工作空间名称
                    - username (str): 工作空间名(用于组织相关的 URL)
                    - role (str): 用户在该工作空间中的角色，如 'OWNER' 或 'MEMBER'
        """
        return self.group.list_workspaces()

    def get_experiment(
            self,
            project: str,
            exp_id: str,
            username: str = ""
    ) -> ApiResponse[Experiment]:
        """
        获取实验信息

        Args:
            project (str): 项目名
            exp_id (str): 实验CUID
            username (str): 工作空间名, 默认为用户个人空间

        Returns:
            ApiResponse[Experiment]:
                - code (int): HTTP 状态码
                - errmsg (str): 错误信息, 仅在请求有错误时非空
                - data (Dict): 实验信息的字典, 包含实验信息
        """
        return self.experiment.get_experiment(
            username=username if username else self.http.username, projname=project, exp_id=exp_id
        )

    def delete_experiment(
            self,
            project: str,
            exp_id: str,
            username: str = ""
    ) -> ApiResponse[None]:
        """
        删除实验

        Args:
            project (str): 项目名
            exp_id (str): 实验CUID
            username (str): 工作空间名, 默认为用户个人空间

        Returns:
            ApiResponse[None]:
                - code (int): HTTP 状态码
                - errmsg (str): 错误信息, 仅在请求有错误时非空
                - data (None): 无数据返回
        """
        return self.experiment.delete_experiment(
            username=username if username else self.http.username, projname=project, exp_id=exp_id
        )

    def list_experiments(
            self,
            project: str,
            username: str = ""
    ) -> ApiResponse[List[Experiment]]:
        """
        获取一个项目下的所有实验

        Args:
            project (str): 项目名
            username (str): 工作空间名, 默认为用户个人空间

        Returns:
            ApiResponse[Experiment]:
                - code (int): HTTP 状态码
                - errmsg (str): 错误信息, 仅在请求有错误时非空
                - data (List[Experiment]): 实验列表, 每个元素包含一个实验的信息
                    - 此实验的 profile 只包含 config (实验自定义配置)
        """
        return self.service.fetch_paginated_api(
            api_func=self.experiment.list_experiments,
            projname=project,
            username=username if username else self.http.username
        )

    def delete_project(
            self,
            project: str,
            username: str = ""
    ) -> ApiResponse[None]:
        """
        删除一个项目

        Args:
            project (str): 项目名
            username (str): 工作空间名, 默认为用户个人空间

        Returns:
            ApiResponse[None]:
                - code (int): HTTP 状态码
                - errmsg (str): 错误信息, 仅在请求有错误时非空
                - data (None): 无数据返回
        """
        return self.project.delete_project(
            username=username if username else self.http.username, project=project
        )

    def list_projects(
            self,
            username: str = "",
            detail: bool = True
    ) -> ApiResponse[List[Project]]:
        """
        获取一个工作空间下的所有项目

        Args:
            username (str): 工作空间名, 默认为用户个人空间
            detail (bool): 是否包含详细统计信息，默认为 True

        Returns:
            ApiResponse[List[Project]]:
                - code (int): HTTP 状态码
                - errmsg (str): 错误信息, 仅在请求有错误时非空
                - data (List[Project]): 项目列表, 每个元素包含一个项目的信息
        """
        return self.service.fetch_paginated_api(
            api_func=self.project.list_projects,
            username=username if username else self.http.username,
            detail=detail
        )

    def get_summary(
            self,
            project: str,
            exp_id: str,
            username: str = ""
    ) -> ApiResponse[Dict]:
        """
        获取实验的概要信息

        Args:
            project (str): 项目名
            exp_id (str): 实验CUID
            username (str): 工作空间名, 默认为用户个人空间

        Returns:
            ApiResponse[Dict]:
                - code (int): HTTP 状态码
                - errmsg (str): 错误信息, 仅在请求有错误时非空
                - data (Dict): 实验的概要信息字典, 包含用户训练各指标的最大最小值, 及其对应步数
        """
        username = username if username else self.http.username
        project_cuid = self.service.get_project_info(username=username, projname=project).data.get("cuid", "")
        exp = self.service.get_exp_info(username=username, project=project, exp_id=exp_id)
        return self.experiment.get_summary(
            exp_id=exp_id,
            pro_id=project_cuid,
            root_exp_id=exp.data.get("rootExpId", ""), 
            root_pro_id=exp.data.get("rootProId", "")
        )

    def get_metrics(
        self,
        exp_id: str,
        keys: Union[str, List[str]],
    ) -> ApiResponse[DataFrame]:
        """
        获取实验的指标数据

        Args:
            exp_id (str): 实验CUID
            keys (str | List[str]): 指标key, 单个字符串或字符串列表

        Returns:
            ApiResponse[DataFrame]: 包含指标数据的响应, 指标数据以 DataFrame 格式返回
            在DataFrame中, 每个key对应两个列, 分别为key和key_timestamp, 表示指标值和时间戳
        """
        if isinstance(keys, str):
            keys = [keys]
        return self.experiment.get_metrics(exp_id, keys)

    def get_exp_summary(self, *args, **kwargs) -> ApiResponse[Dict]:
        """
        获取实验的概要信息
        @deprecated, 请使用 `get_experiment_summary`

        Returns:
            ApiResponse[Dict]: 包含实验概要信息的响应
        """
        return self.get_summary(*args, **kwargs)

    def list_project_exps(self, *args, **kwargs) -> ApiResponse[List[Experiment]]:
        """
        获取一个项目下的所有实验
        @deprecated, 请使用 `list_experiments`

        Returns:
            ApiResponse[List[Experiment]]: 包含实验列表的响应
        """
        return self.list_experiments(*args, **kwargs)
